require "behaviours/chaseandattack"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/follow"
require "behaviours/standstill"
require "behaviours/runaway"
require "behaviours/chattynode"

local MIN_FOLLOW_DIST = 2
local MAX_FOLLOW_DIST = 9
local TARGET_FOLLOW_DIST = (MAX_FOLLOW_DIST+MIN_FOLLOW_DIST)/2
local MAX_CHASE_TIME = 10
local MAX_CHASE_DIST = 30
local RUN_AWAY_DIST = 5
local STOP_RUN_AWAY_DIST = 8
local SEE_TREE_DIST = 15
local KEEP_CHOPPING_DIST = 10
local SEE_ROCK_DIST = 12
local KEEP_MINING_DIST = 8
local GIRL = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH

local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function KeepChoppingAction(inst)
    return inst.components.follower.leader and inst.components.follower.leader:GetDistanceSqToInst(inst) <= KEEP_CHOPPING_DIST*KEEP_CHOPPING_DIST
end

local function StartChoppingCondition(inst)
    return inst.components.follower.leader and inst.components.follower.leader.sg and inst.components.follower.leader.sg:HasStateTag("chopping")
end

local function FindTreeToChopAction(inst)
    local target = FindEntity(inst, SEE_TREE_DIST, function(item) return item.components.workable and item.components.workable.action == ACTIONS.CHOP end)
    if target then
        return BufferedAction(inst, target, ACTIONS.CHOP)
    end
end

local function KeepMiningAction(inst)
    return inst.components.follower.leader and inst.components.follower.leader:GetDistanceSqToInst(inst) <= KEEP_MINING_DIST*KEEP_MINING_DIST
end

local function StartMiningCondition(inst)
    return inst.components.follower.leader and inst.components.follower.leader.sg and inst.components.follower.leader.sg:HasStateTag("mining")
end

local function FindRockToMineAction(inst)
    local target = FindEntity(inst, SEE_ROCK_DIST, function(item) return item.components.workable and item.components.workable.action == ACTIONS.MINE end)
    if target then
        return BufferedAction(inst, target, ACTIONS.MINE)
    end
end

local function CraftChromeSword(inst)
	if not inst.sg:HasStateTag("crafting") then inst.sg:GoToState("craftchromesword") end
end

local GoogleIRLBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

function GoogleIRLBrain:OnStart()
    
    local root = PriorityNode({
		IfNode(function() return not self.inst.sg:HasStateTag("transforming") end, "everything", PriorityNode{
			IfNode(function() return not self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) and not self.inst.components.health:IsDead() end, "craft chromesword",
				DoAction(self.inst, CraftChromeSword)),
			WhileNode( function() return self.inst.components.combat.target == nil or not self.inst.components.combat:InCooldown() end, "AttackMomentarily",
				ChaseAndAttack(self.inst, MAX_CHASE_TIME, MAX_CHASE_DIST)),
			WhileNode( function() return self.inst.components.combat.target and self.inst.components.combat:InCooldown() end, "Dodge",
				RunAway(self.inst, function() return self.inst.components.combat.target end, RUN_AWAY_DIST, STOP_RUN_AWAY_DIST)),
			RunAway(self.inst, function(guy) return guy.components.combat and guy.components.combat.target == self.inst end, RUN_AWAY_DIST, STOP_RUN_AWAY_DIST ),
			IfNode(function() return StartChoppingCondition(self.inst) end, "chop", 
				WhileNode(function() return KeepChoppingAction(self.inst) end, "keep chopping",
					LoopNode{ChattyNode(self.inst, GIRL.chop_tree,
						DoAction(self.inst, FindTreeToChopAction))})),
			IfNode(function() return StartMiningCondition(self.inst) end, "mine", 
				WhileNode(function() return KeepMiningAction(self.inst) end, "keep mining",
					LoopNode{ChattyNode(self.inst, GIRL.mine_rock,
						DoAction(self.inst, FindRockToMineAction))})),
			Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
			FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn)})
    }, .5)
    
    self.bt = BT(self.inst, root)
end

return GoogleIRLBrain