require "behaviours/follow"
require "behaviours/faceentity"

local MIN_FOLLOW_DIST = 0
local MAX_FOLLOW_DIST = 7
local TARGET_FOLLOW_DIST = 7

local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function WakeUp(inst)
	if inst.sg:HasStateTag("sleeping") then
		return inst.sg:GoToState("wakeup")
	end
end

local function Sleep(inst)
	if not inst.sg:HasStateTag("sleeping") and not inst.sg:HasStateTag("open") then
		return inst.sg:GoToState("sleep")
	end
end

local tinyboxtimdumbBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

function tinyboxtimdumbBrain:OnStart()
    local clock = GetClock()
	local root = PriorityNode({
		WhileNode(function() return self.inst.components.chatterbox.convo == true or self.inst.sg:HasStateTag("talking") end, "IsTalking",
			PriorityNode{FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn)},
		.5),
		WhileNode(function() return GetWorld():IsCave() end, "IsCave",
			PriorityNode{Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST/1.5, MAX_FOLLOW_DIST/1.5)},
		.5),
		WhileNode(function() return clock and not clock:IsNight() end, "NotNight", DoAction(self.inst, function() return WakeUp(self.inst) end, "WakeUp", true)),
		WhileNode(function() return clock and clock:IsDay() end, "IsDay",
			PriorityNode{Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST)},
		.5),
		WhileNode(function()return clock and clock:IsDusk() end, "IsDusk",
			PriorityNode{Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST/1.2, MAX_FOLLOW_DIST/1.2)},
		.5),
		WhileNode(function() return clock and clock:IsNight() and clock:GetMoonPhase() == "full" end, "IsFullMoon",
			PriorityNode{Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST/1.7, MAX_FOLLOW_DIST/1.7)},
		.5),
		WhileNode(function() return clock and clock:IsNight() end, "IsNight",
			PriorityNode{Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
			DoAction(self.inst, function() return Sleep(self.inst) end, "Sleep", true)},
		.5),
		FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn)
	}, .5)
    self.bt = BT(self.inst, root)
end

return tinyboxtimdumbBrain