local ChatterBox = Class(function(self, inst)
    self.inst = inst
	self.conditionfn = nil
	self.itemtracking = {}
	self.convo_target = nil
	self.run_convo = false
	self.convo_maximum = 0
	self.convo_progress = 0
	self.convotask_inst = nil
	self.convotask_player = nil
	self.convo_markstate = "Markiplier"
end)

function ChatterBox:SetCondition(fn)
	self.conditionfn = fn
end

function ChatterBox:SetIdleChat(fn)
	self.idlechatfn = fn
end

function ChatterBox:SetItemTracker(itemtable)
	self.itemtracking = itemtable
end

function ChatterBox:EnableConversing(min_time, max_time, chance, range, inststring, instname)
	self:Converse(min_time, max_time, chance, range, inststring, instname)
	self:CheckConvoTarget(range, inststring)
end

function ChatterBox:Condition()
	if self.conditionfn then
		return self:conditionfn()
	else
		return true
	end
end

function ChatterBox:CheckSurroundings()
	if GetWorld():IsCave() then
		return "cave"
	else
		if GetClock():IsNight() and GetClock():GetMoonPhase() == "full" then
			return "fullmoon"
		else
			return GetClock():GetPhase()
		end
	end
end

function ChatterBox:CheckPlayersAction(player)
	if player.sg:HasStateTag("chopping") then
		return "chop"
	elseif player.sg:HasStateTag("mining") then
		return "mine"
	elseif player.sg:HasStateTag("hammering") then
		return "hammer"
	elseif player.sg:HasStateTag("operating") then
		return "operate"
	else
		return false
	end
end

function ChatterBox:CheckItem(range)
	if self.itemtracking then
		local close_prefabs = {}
		local pos = self.inst:GetPosition()
		for k,v in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, range)) do
			for i,o in pairs(self.itemtracking) do
				for a,b in pairs(o) do
					print(b)
					if b == v.prefab then
						table.insert(close_prefabs, i)
					end
				end
			end
		end
		if #close_prefabs ~= 0 then
			return close_prefabs[math.random(1,#close_prefabs)]
		else
			return false
		end
	else
		return false
	end
end

function ChatterBox:FindPlayer(range)
	local players = {}
	local pos = self.inst:GetPosition()
	for k,v in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, range)) do
		if v:HasTag("player") or (v.components.chatterbox and v.components.chatterbox.run_convo ~= true) and v ~= self.inst then
			table.insert(players, v)
		end
	end
	if #players ~= 0 then
		return players[math.random(1,#players)]
	else
		return false
	end
end

function ChatterBox:IdleChat(min_time, max_time, chance)
	self.inst:DoTaskInTime(math.random(min_time, max_time), function()
		if self:Condition() then
			if math.random(1,100) <= chance then
				if self.idlechatfn then self:idlechatfn() end
			end
		end
		self:IdleChat(min_time, max_time, chance)
	end)
end

function ChatterBox:Encourage(min_time, max_time, chance, playerprefab, range, inststring)
	self.inst:DoTaskInTime(math.random(min_time, max_time), function()
		if self:Condition() then
			if math.random(1,100) <= chance then
				local PlayerHere = false
				local Player = nil
				local pos = self.inst:GetPosition()
				local PlayerTable = {}
				for k,v in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, range)) do
					if v:HasTag("player") then
						if playerprefab ~= nil then
							if v.prefab == playerprefab then
								PlayerHere = true
								Player = v
							end
						else
							PlayerHere = true
							table.insert(PlayerTable, v)
							if #PlayerTable > 1 then
								Player = math.random(1,#PlayerTable)
							else Player = v end
						end
					end
				end
				if PlayerHere == true then
					local action = self:CheckPlayersAction(Player)
					if action ~= false then
						self.inst.components.talker:Say(GetRandomItem(inststring[action]))
					end
				end
			end
		end
		self:Encourage(min_time, max_time, chance, playerprefab, range, inststring)
	end)
end

function ChatterBox:LookForItems(min_time, max_time, chance, range, inststring)
	self.inst:DoTaskInTime(math.random(min_time, max_time), function()
		if self:Condition() then
			if math.random(1,100) <= chance then
				local item = self:CheckItem(range)
				if item ~= false then
					self.inst.components.talker:Say(GetRandomItem(inststring[item]))
				end
			end
		end
		self:LookForItems(min_time, max_time, chance, range, inststring)
	end)
end

function ChatterBox:Converse(min_time, max_time, chance, range, inststring, instname)
	self.inst:DoTaskInTime(math.random(min_time, max_time), function()
		if self:Condition() then
			if math.random(1,100) <= chance then
				local player = self:FindPlayer(range)
				if player ~= false then
					self.convo_target = player
					if player.prefab == "markiplier" then
						self.convo_markstate = player.MarkState
					end
					self.run_convo = true
					self:Conversation(self.inst, player, inststring, instname)
				end
			end
		end
		self:Converse(min_time, max_time, chance, range, inststring, instname)
	end)
end

function ChatterBox:Conversation(inst, player, inststring, instname)
	local speech = inststring.converse
	local target = player.prefab
	if target == "markiplier" then target = string.lower(self.convo_markstate) end
	if not player.components.talker then target = "wes" end
	if not speech[target] then target = "generic" end
	local convo = math.random(1,#speech[target])
	local playerspeech = speech[target][convo]["player"]
	local instspeech = speech[target][convo][instname]
	self.convo_maximum = #(speech[target][convo][instname])
	self.convo_progress = 1
	self.convotask_inst = inst:DoPeriodicTask(5, function()
		inst.components.talker:Say(instspeech[self.convo_progress])
	end)
	player:DoTaskInTime(2.5, function()
		if player.components.chatterbox then
			if player.components.chatterbox.run_convo ~= true then
				player.components.chatterbox.run_convo = true
			else
				self.inst:DoTaskInTime(0.5, function() self:Cancel_Convo(inststring) end)
			end
		end
		self.convotask_player = player:DoPeriodicTask(5, function()
			if target ~= "wes" then
				player.components.talker:Say(playerspeech[self.convo_progress])
			else
				player.sg:GoToState("mime")
			end
			if self.convo_progress >= self.convo_maximum then
				self:Cancel_Convo()
			else
				self.convo_progress = self.convo_progress + 1
			end
		end)
	end)
end

function ChatterBox:CheckConvoTarget(range, inststring)
	self.inst:DoTaskInTime(1, function()
		if self.run_convo ~= false then
			local close_enough = false
			local pos = self.inst:GetPosition()
			for k,v in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, range)) do
				if v:HasTag("player") or v.components.chatterbox then
					if v == self.convo_target then
						if v.prefab ~= "markiplier" then
							close_enough = true
						else
							if v.MarkState == self.convo_markstate then
								close_enough = true
							end
						end
					end
				end
			end
			if close_enough ~= true then
				self:Cancel_Convo(inststring)
			end
		end
		self:CheckConvoTarget(range, inststring)
	end)
end

function ChatterBox:Cancel_Convo(inststring)
	if self.run_convo ~= false then
		self.inst.components.talker:ShutUp()
		if inststring then self.inst.components.talker:Say(GetRandomItem(inststring["cancelconvo"])) end
		self.run_convo = false
		self.convo_maximum = 0
		self.convo_progress = 0
		--[[if self.convo_target.components.chatterbox and self.convo_target.components.chatterbox.run_convo == true then
			self.convo_target.components.chatterbox.run_convo = false
		end]]
		self.convo_target = nil
		if self.convotask_inst ~= nil then
			self.convotask_inst:Cancel()
			self.convotask_inst = nil
		end
		if self.convotask_player ~= nil then
			self.convotask_player:Cancel()
			self.convotask_player = nil
		end
	end
end

return ChatterBox