local G = GLOBAL
local require = G.require
local Action = G.Action
local ACTIONS = G.ACTIONS
local STRINGS = G.STRINGS
require "class"
require "bufferedaction"

local markimod_actions = {
	CYCLE_STATE = Action(1, true),
	DRINK = Action(2, true, true),
	OPERATE = Action(),
	GIRL_TRANSFORM_ON = Action(1, false, false),
	GIRL_TRANSFORM_OFF = Action(1, false, false),
	GIRL_LIGHT_ON = Action(1, false, true),
	GIRL_LIGHT_OFF = Action(1, false, true)
}

for k,v in pairs(markimod_actions) do
	ACTIONS[k] = v
	ACTIONS[k].str = STRINGS.ACTIONS[k]
	ACTIONS[k].id = k
end

ACTIONS.CYCLE_STATE.fn = function(act)
	local obj = act.target or act.invobject
	if obj.components.statecycler:CanCycle() then
		obj.components.statecycler:Cycle()
	end
end

ACTIONS.GIRL_LIGHT_ON.fn = function(act)
	if not act.invobject then
		if act.target.components.girllight:CanCycle() then
			act.target.components.girllight:Cycle()
			return true
		end
	end
end

ACTIONS.GIRL_LIGHT_OFF.fn = function(act)
	if not act.invobject then
		if act.target.components.girllight:CanCycle() then
			act.target.components.girllight:Cycle()
			return true
		end
	end
end

ACTIONS.GIRL_TRANSFORM_ON.fn = function(act)
	local obj = act.target or act.invobject
	if not (obj.components.inventoryitem and obj.components.inventoryitem.owner) then
		if obj.components.girltransformer:CanCycle() then
			obj.components.girltransformer:Cycle()
			return true
		end
	end
end

ACTIONS.GIRL_TRANSFORM_OFF.fn = function(act)
	local obj = act.target or act.invobject
	if not (obj.components.inventoryitem and obj.components.inventoryitem.owner) then
		if obj.components.girltransformer:CanCycle() then
			obj.components.girltransformer:Cycle()
			return true
		end
	end
end

ACTIONS.DRINK.fn = function(act)
    local obj = act.target or act.invobject
    if act.doer.components.eater and obj and obj.components.edible then
    	return act.doer.components.eater:Eat(obj) 
    end
end
local Edible = GLOBAL.require "components/edible"
function Edible:CollectInventoryActions(doer, actions, right)
	local eat = ACTIONS.EAT
	if self.inst:HasTag("drink") then
		eat = ACTIONS.DRINK
	end
	if GLOBAL.ROG then
		if doer.components.eater and doer.components.eater:IsValidFood(self.inst) and doer.components.eater:AbleToEat(self.inst) then
			if not self.inst.components.equippable or right then
				table.insert(actions, eat)
			end
		end
	else
		if doer.components.eater and doer.components.eater:CanEat(self.inst) then
			if not self.inst.components.equippable or right then
				table.insert(actions, eat)
			end
		end
	end
end

ACTIONS.OPERATE.fn = function(act)
    if act.target.components.workable and act.target.components.workable.action == ACTIONS.OPERATE then
        local numworks = 1
		
        if act.invobject and act.invobject.components.tool then
            numworks = act.invobject.components.tool:GetEffectiveness(ACTIONS.OPERATE)
        elseif act.doer and act.doer.components.worker then
            numworks = act.doer.components.worker:GetEffectiveness(ACTIONS.OPERATE)
        end
        act.target.components.workable:WorkedBy(act.doer, numworks)
    end
    return true
end