local G = GLOBAL
local ActionHandler = G.ActionHandler
local ACTIONS = G.ACTIONS
local State = G.State
local EventHandler = G.EventHandler
local TimeEvent = G.TimeEvent
local FRAMES = G.FRAMES
local TheInput = G.TheInput

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.OPERATE, 
function(inst) 
	if not inst.sg:HasStateTag("preoperate") then 
		if inst.sg:HasStateTag("operating") then
			return "operate"
		else
			return "operate_start"
		end
	end 
end))

AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.GIRL_LIGHT_ON, "doshortaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.GIRL_LIGHT_OFF, "doshortaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.GIRL_TRANSFORM_ON, "doshortaction"))
AddStategraphActionHandler("wilson", ActionHandler(ACTIONS.GIRL_TRANSFORM_OFF, "doshortaction"))

AddStategraphState("wilson", State{
	name = "operate_start",
	tags = {"preoperate", "working"},
	onenter = function(inst)
		inst.components.locomotor:Stop()
		inst.AnimState:PlayAnimation("pickaxe_pre")
	end,
	
	events = {EventHandler("animover", function(inst) inst.sg:GoToState("operate") end)}
})
    
AddStategraphState("wilson", State{
	name = "operate",
	tags = {"preoperate", "operating", "working"},
	onenter = function(inst)
		inst.sg.statemem.action = inst:GetBufferedAction()
		inst.AnimState:PlayAnimation("pickaxe_loop")
		if math.random(1,100) <= 30 then
			local player = inst.prefab
			if not inst.components.talker then player = nil else
				if not G.STRINGS.MARKIMODSTRINGS.OPERATE.player then player = "default"
				elseif inst.prefab == "markiplier" then player = (string.lower(inst.MarkState)) end
			end
			if player ~= nil then inst.components.talker:Say(G.GetRandomItem(G.STRINGS.MARKIMODSTRINGS.OPERATE.player)) end
		end
	end,

	timeline = {
		TimeEvent(9*FRAMES, function(inst) 
			inst:PerformBufferedAction() 
			inst.sg:RemoveStateTag("preoperate") 
			inst.SoundEmitter:PlaySound("dontstarve/wilson/hit")
		end),
		
		TimeEvent(14*FRAMES, function(inst)
		
			if (TheInput:IsControlPressed(G.CONTROL_SECONDARY) or
			   TheInput:IsControlPressed(G.CONTROL_ACTION) or TheInput:IsControlPressed(G.CONTROL_CONTROLLER_ALTACTION)) and 
				inst.sg.statemem.action and 
				inst.sg.statemem.action.target and 
				inst.sg.statemem.action.target:IsActionValid(inst.sg.statemem.action.action, true) and 
				inst.sg.statemem.action.target.components.workable then
					inst:ClearBufferedAction()
					inst:PushBufferedAction(inst.sg.statemem.action)
			end
		end),
	},
	events = {
		EventHandler("unequip", function(inst) inst.sg:GoToState("idle") end ),
		EventHandler("animover", function(inst) 
			inst.AnimState:PlayAnimation("pickaxe_pst") 
			inst.sg:GoToState("idle", true)
		end )
	}      
})