local SwearControl = MarkMod.SwearControl
local PauseScreen = require "screens/pausescreen"

local assets = {
	Asset( "ANIM", "anim/player_basic.zip" ),
	Asset( "ANIM", "anim/player_idles_shiver.zip" ),
	Asset( "ANIM", "anim/player_actions.zip" ),
	Asset( "ANIM", "anim/player_actions_axe.zip" ),
	Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
	Asset( "ANIM", "anim/player_actions_shovel.zip" ),
	Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
	Asset( "ANIM", "anim/player_actions_eat.zip" ),
	Asset( "ANIM", "anim/player_actions_item.zip" ),
	Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
	Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
	Asset( "ANIM", "anim/player_actions_fishing.zip" ),
	Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
	Asset( "ANIM", "anim/player_bush_hat.zip" ),
	Asset( "ANIM", "anim/player_attacks.zip" ),
	Asset( "ANIM", "anim/player_idles.zip" ),
	Asset( "ANIM", "anim/player_rebirth.zip" ),
	Asset( "ANIM", "anim/player_jump.zip" ),
	Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
	Asset( "ANIM", "anim/player_teleport.zip" ),
	Asset( "ANIM", "anim/wilson_fx.zip" ),
	Asset( "ANIM", "anim/player_one_man_band.zip" ),
	Asset( "ANIM", "anim/shadow_hands.zip" ),
	Asset( "SOUND", "sound/sfx.fsb" ),
	Asset( "SOUND", "sound/maxwell.fsb" ),
	Asset( "ANIM", "anim/beard.zip" ),
	
	Asset("SOUND", "sound/markiplier_sounds_bank00.fsb"),
	Asset( "ANIM", "anim/googleirl.zip"),
	Asset( "ANIM", "anim/googleirl_placer.zip"),
	Asset( "ANIM", "anim/googleirl_transform.zip"),
	Asset("ATLAS", "images/inventoryimages/googleirl.xml")
}

local itemassets = {
	Asset( "ANIM", "anim/googleirl_transform.zip"),
	Asset("ATLAS", "images/inventoryimages/googleirlitem.xml")
}

local prefabs = {
	"chromesword",
	"googleirlitem"
}

local itemprefabs = {
	"googleirl"
}

SetSharedLootTable( 'googleirl', {
	{'googlepiece', 1},
	{'gears', 0.3},
	{'gears', 0.2}
})

local function retargetfn(inst)
    return FindEntity(inst, TUNING.PIG_TARGET_DIST, function(guy)
		if not guy.LightWatcher or guy.LightWatcher:IsInLight() then
			return (guy:HasTag("monster") and not guy:HasTag("player")) and guy.components.health and not guy.components.health:IsDead() and inst.components.combat:CanTarget(guy) and not 
			(inst.components.follower.leader ~= nil and guy:HasTag("abigail"))
		end
	end)
end

local function keeptarget(inst, target)
    return inst.components.combat:CanTarget(target) and (not target.LightWatcher or target.LightWatcher:IsInLight()) and not (target.sg and target.sg:HasStateTag("transform") )
end

local function WeaponDropped(weapon)
	if math.random(1,100) <= 80 then
		weapon:Remove()
	else
		weapon.components.inventoryitem.candrop = true
		weapon.components.inventoryitem:SetOnDroppedFn(function()end)
	end
end

local function EquipWeapon(inst)
    if inst.components.inventory and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
		local weapon = SpawnPrefab("chromesword")
        weapon.entity:AddTransform()
        weapon.persists = false
		weapon.components.inventoryitem.candrop = false
        weapon.components.inventoryitem:SetOnDroppedFn(WeaponDropped)
        inst.components.inventory:Equip(weapon)
    end
end

local function onkilledbyother(inst, attacker)
	if inst.VeryVeryDead == nil then
		inst.SoundEmitter:PlaySound("dontstarve/common/blackpowder_fuse_LP", "hiss")
		inst:DoTaskInTime(1.5, function()
			if inst.Light:IsEnabled() == true then
				if inst.components.fueled then
					inst.components.fueled:StopConsuming()        
				end
				inst.Light:Enable(false)
			end
			local pos = Vector3(inst.Transform:GetWorldPosition())
			inst.SoundEmitter:KillSound("hiss")
			inst.SoundEmitter:PlaySound("dontstarve/creatures/slurtle/mound_explode")
			local explode = SpawnPrefab("explode_small")
			local pos = inst:GetPosition()
			explode.Transform:SetPosition(pos.x, pos.y, pos.z)
			explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
			explode.AnimState:SetLightOverride(1)
			GetClock():DoLightningLighting()    
			GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, .5, 40)
			local pos = Vector3(inst.Transform:GetWorldPosition())
			local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 3)
			for k,v in pairs(ents) do
				if v.components.workable and not v:HasTag("busy") then
					v.components.workable:WorkedBy(inst, 10)
				elseif v.components.burnable and not v.components.fueled then
					if math.random(1,100) <= 10 then v.components.burnable:Ignite() end
				end
				if v.components.combat and v ~= inst then
					v.components.combat:GetAttacked(inst, 200, nil)
				end
				v:PushEvent("explosion", {explosive = inst})
			end
		end)
		inst.VeryVeryDead = true
	end
end

local function OnAttacked(inst, data)
	inst.components.combat:SuggestTarget(data.attacker)
end

local function getstatus(inst)
	if inst.components.girltransformer.currentstate == "off" then
	return "ITEM" end
end

local function ExaminationDetected(inst, player)
	if not DST then
		if MarkMod.PooFish then
			if inst.components.girltransformer.currentstate == "on" then
				inst:DoTaskInTime(2, function() inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.poo_the_fishes)
					player:DoTaskInTime(2, function() player.components.talker:Say("What?") end)
					inst:DoTaskInTime(3, function() inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH.search_poothefishes)
						inst:DoTaskInTime(2, function()
							TheFrontEnd:PushScreen(PauseScreen())
							VisitURL("http://goo.gl/8r3Yuy")
						end)
					end)
				end)
			end
		end
	end
end

local function GetFuelPercent(inst)
	return tostring(inst.components.fueled:GetPercent() * 100)
end

local function turnon(inst, dark)
	if inst.components.girllight:State() == "on"  and inst.components.girltransformer:State() == "on" then
		print(inst.components.girltransformer:State())
		local GIRL = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH
		if inst.Light:IsEnabled() == false then
			if not inst.components.fueled:IsEmpty() then
				if inst.components.fueled then
					inst.components.fueled:StartConsuming()        
				end
				inst.Light:Enable(true)
				inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_on")
				inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_LP", "loop")
				if dark then
					inst.components.talker:Say(GIRL.fuel_increased_enable.."\n"..GIRL.current_level..GetFuelPercent(inst)..GIRL.percent)
				else
					inst.components.talker:Say(GIRL.enable_light.."\n"..GIRL.current_level..GetFuelPercent(inst)..GIRL.percent)
				end
			else
				inst.components.talker:Say(GIRL.failed_to_enable.."\n"..GIRL.already_on)
			end
		else
			inst.components.talker:Say(GIRL.failed_to_enable.."\n"..GIRL.fuel_gone)
		end
	end
end

local function turnoff(inst, depleted)
	local GIRL = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH
	if inst.Light:IsEnabled() == true then
		if inst.components.fueled then
			inst.components.fueled:StopConsuming()        
		end
		inst.Light:Enable(false)
		inst.SoundEmitter:KillSound("loop")
		inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_off")
		if depleted then
			inst.components.talker:Say(GIRL.fuel_gone_two.."\n"..GIRL.disable_light)
		else
			inst.components.talker:Say(GIRL.disable_light.."\n"..GIRL.current_level..GetFuelPercent(inst)..GIRL.percent)
		end
	else
		inst.components.talker:Say(GIRL.disable_light.."\n"..GIRL.already_off)
	end
end

local function nofuel(inst)
    if inst.Light:IsEnabled() == true then turnoff(inst, true) end
end

local function CheckFullMoon()
	if ROG then if GetClock():IsNight() and GetClock():GetMoonPhase() == "full" then return true else return false end
	else return false end
end

local function CheckDark()
	if GetWorld():IsCave() or (GetClock():IsNight() and CheckFullMoon() == false) then
	return true end
end

local function CheckLight()
	if not GetWorld():IsCave() then if GetClock():IsDay() or GetClock():IsDusk() or CheckFullMoon() == true then
	return true end end
end

local function takefuel(inst)
	local GIRL = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH
	inst.components.talker:Say(GIRL.current_level..GetFuelPercent(inst)..GIRL.percent)
    if inst:checkdark() and inst.Light:IsEnabled() == false then
        turnon(inst, true)
    end
end

local function fuelupdate(inst)
    local fuelpercent = inst.components.fueled:GetPercent()
    inst.Light:SetIntensity(Lerp(0.4, 0.6, fuelpercent))
    inst.Light:SetRadius(Lerp(3, 5, fuelpercent))
    inst.Light:SetFalloff(.9)
end

local function ondrop(inst)
	inst.sg:GoToState("idle")
end

local function makeinventoryitem(inst)
	local fuel = inst.components.fueled:GetPercent()
	local function WeaponDurability() if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then return inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS).components.finiteuses:GetUses() else return 1 end end
	local weapondur = WeaponDurability()
	local health = inst.components.health.currenthealth
	local light = inst.components.girllight.currentstate
	local item = SpawnPrefab("googleirlitem")
	item.Transform:SetPosition(inst:GetPosition():Get())
	item:addvalues(fuel, weapondur, health, light)
	inst:Remove()
end

local function frominventoryvalues(inst, fuel, weapondur, health, light)
	inst.AnimState:SetBank("googleirl_transform")
	inst.AnimState:SetBuild("googleirl_transform")
	inst.sg:GoToState("transformup")
	inst.components.fueled:SetPercent(fuel)
	inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS).components.finiteuses:SetUses(weapondur)
	inst.components.health.currenthealth = health
	inst.components.girllight.currentstate = light
end

local function onsave(inst, data)
end

local function onload(inst, data)
	if inst.Light:IsEnabled() == true then turnoff(inst) end
	if inst.components.girltransformer.currentstate == "off" then
		if not inst.components.inventoryitem then inst:makeinventoryitem(inst) end
	else
		if inst.components.inventoryitem then inst:removeinventoryitem(inst) end
		if inst:checkdark() then
			if inst.Light:IsEnabled() == false then turnon(inst) end
		elseif inst:checklight() then
			if inst.Light:IsEnabled() == true then turnoff(inst) end
		end
	end
end

local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local physics = inst.entity:AddPhysics()
	local sound = inst.entity:AddSoundEmitter()
	local lightwatch = inst.entity:AddLightWatcher()
	lightwatch:SetLightThresh(.075)
	lightwatch:SetDarkThresh(.05)
	
	inst.AnimState:SetBank("wilson")
	inst.AnimState:SetBuild("googleirl")
	inst.AnimState:PlayAnimation("idle_loop")
	
	inst.Transform:SetFourFaced()
	MakeCharacterPhysics(inst, 10, 1.5)
	inst.Physics:ClearCollisionMask()
	inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
	inst.Physics:CollidesWith(COLLISION.WORLD)
	inst.Physics:CollidesWith(COLLISION.OBSTACLES)
	inst.Physics:CollidesWith(COLLISION.CHARACTERS)
	
	inst:AddComponent("talker")
	inst.components.talker.fontsize = 28
	inst.components.talker.font = TALKINGFONT
	inst.components.talker.colour = Vector3(140/255, 200/255, 215/255)
	inst.components.talker.offset = Vector3(0,-400,0)
	
	inst.turnon = turnon
	inst.turnoff = turnoff
	inst.checkdark = CheckDark
	inst.checklight = CheckLight
	inst.makeinventoryitem = makeinventoryitem
	inst.frominventoryvalues = frominventoryvalues
	
	inst:AddComponent("girltransformer")
	inst:AddComponent("girllight")
	
	inst.entity:AddLight()
    inst.Light:SetColour(140/255, 200/255, 215/255)
	inst.Light:Enable(false)
	inst:AddComponent("fueled")
	inst.components.fueled.fueltype = "CAVE"
    inst.components.fueled:InitializeFuelLevel(TUNING.LANTERN_LIGHTTIME * 0.75)
    inst.components.fueled:SetDepletedFn(nofuel)
    inst.components.fueled:SetUpdateFn(fuelupdate)
    inst.components.fueled.ontakefuelfn = takefuel
    inst.components.fueled.accepting = true
	inst.components.fueled:SetPercent(0.25)
    fuelupdate(inst)
	
	inst:DoPeriodicTask(0.1, function() if inst.Light:IsEnabled() == true then
		if inst:checklight() then turnoff(inst) end
	elseif inst.Light:IsEnabled() == false and inst.components.girllight:State() == "on" then
		if inst:checkdark() then if inst.components.fueled:GetPercent() ~= 0 then turnon(inst) end end
	end end)
	
	local commonstringtable = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH
	inst:AddComponent("chatterbox")
	inst.components.chatterbox:SetCondition(function() if not inst.sg:HasStateTag("working") and inst.components.chatterbox.run_convo == false and not inst.components.inventoryitem then return true else return false end end)
	inst.components.chatterbox:SetIdleChat(function()
		if inst.components.chatterbox:CheckSurroundings() ~= "cave" then
			if math.random(1,100) <= 50 then
				local Time = inst.components.chatterbox:CheckSurroundings()
				if commonstringtable[Time] then
					if Time == "fullmoon" then
						if math.random(1,100) <= 20 then
						inst.components.talker:Say(GetRandomItem(commonstringtable["night"])) else
						inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
					else inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
				else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
			else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
		else
			if math.random(1,100) <= 40 then inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"]))
			else inst.components.talker:Say(GetRandomItem(commonstringtable["cave_chat"])) end
		end
	end)
	inst.components.chatterbox:SetItemTracker({
		giants = {"moose", "dragonfly", "deerclops", "bearger"},
		teleportatopieces = {"teleportato_base", "teleportato_crank", "teleportato_ring", "teleportato_potato", "teleportato_box"},
		pig_king = {"pigking", "pigking"}
	})
	inst.components.chatterbox:EnableConversing(55, 140, 60, 25, commonstringtable, "girl")
	inst.components.chatterbox:IdleChat(20, 50, 60)
	inst.components.chatterbox:LookForItems(15, 30, 50, 80, commonstringtable)
	
	inst:AddComponent("inventory")
	EquipWeapon(inst)
	inst.AnimState:Show("ARM_carry") 
	inst.AnimState:Hide("ARM_normal")
	if ROG then inst:AddComponent("moisture") end
	inst:AddComponent("sanity")
	inst.components.sanity:SetMax(50)
	inst.components.sanity.current = 0
	inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * -100)
	
	inst:AddComponent("locomotor")
	inst.components.locomotor.walkspeed = (TUNING.WILSON_WALK_SPEED * 0.9)
	inst.components.locomotor.runspeed = (TUNING.WILSON_RUN_SPEED * 0.9)
	inst.components.locomotor:SetTriggersCreep(false)
	--inst.components.locomotor.pathcaps = { ignorecreep = true }

	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("googleirl.tex")
	
	inst:SetStateGraph("SGgoogleirl")
	inst:AddComponent("follower")
	if TheSim:FindFirstEntityWithTag("Darkiplier") then
	inst.components.follower:SetLeader(TheSim:FindFirstEntityWithTag("Darkiplier")) end
	inst:AddComponent("inspectable")
	inst.components.inspectable.description = ExaminationDetected
	inst:AddComponent("lootdropper")
	--inst.components.lootdropper:SetChanceLootTable('googleirl')
	inst:AddTag("notraptrigger")
	inst:AddTag("GoogleIRL")
	inst:AddTag("monster")
	
	inst:AddComponent("health")
	inst.components.health:SetMaxHealth (1000)
	inst.components.health:StartRegen(1, 5)
	inst.components.health.canmurder = false
	
	inst:AddComponent("combat")
	inst.components.combat:SetDefaultDamage(10)
	inst.components.combat.damagemultiplier = 1
	inst.components.combat:SetAttackPeriod(3)
	inst.components.combat:SetRange(3)
	inst.components.combat:SetRetargetFunction(1, retargetfn)
	inst.components.combat.onkilledbyother = onkilledbyother
	inst.components.combat:SetKeepTargetFunction(keeptarget)
	inst:ListenForEvent("attacked", OnAttacked)

    inst.OnLoad = onload 
	inst.OnSave = onsave
	
	local brain = require "brains/googleirlbrain"
	inst:SetBrain(brain)
	
    return inst
end

local function addvalues(inst, fuel, weapondur, health, light)
	inst.fuel = fuel
	inst.weapondur = weapondur
	inst.health = health
	inst.light = light
end

local function removeinventory(inst)
	local google = SpawnPrefab("googleirl")
	google.Transform:SetPosition(inst:GetPosition():Get())
	google:frominventoryvalues(inst.fuel, inst.weapondur, inst.health, inst.light)
	inst:Remove()
end

local function itemonsave(inst, data)
	data.fuel = inst.fuel
	data.weapondur = inst.weapondur
	data.health = inst.health
	data.light = inst.light
end

local function itemonload(inst, data)
	if data then
		inst.fuel = data.fuel or 0.25
		inst.weapondur = data.weapondur or TUNING.NIGHTSWORD_USES
		inst.health = data.health or 1000
		inst.light = data.light or "on"
	end
end

local function itemfn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	anim:SetBank("googleirl_transform")
	anim:SetBuild("googleirl_transform")
	anim:PlayAnimation("collapsed")
	MakeInventoryPhysics(inst)
	inst:AddComponent("inspectable")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/googleirlitem.xml"
	inst:AddComponent("girltransformer")
	inst.components.girltransformer.currentstate = "off"
	inst.fuel = nil
	inst.weapondur = nil
	inst.health = nil
	inst.light = nil
	inst.transformer = nil
	inst.addvalues = addvalues
	inst.removeinventory = removeinventory
    inst.OnLoad = itemonload 
	inst.OnSave = itemonsave
	return inst
end

return  Prefab( "common/monsters/googleirl", fn, assets, prefabs),
		Prefab( "common/googleirlitem", itemfn, itemassets, itemprefabs),
		MakePlacer("common/googleirl_placer", "googleirl_placer", "googleirl_placer", "idle_loop") 