--[[local function dotConfig(_OddsMin, _OddsMax, _ThresMin, _ThresMax)
   return {
      oddsMin  = _OddsMin,  --our odds when threshold is <= threshold1
      oddsMax  = _OddsMax,  --our odds when threshold is >= threshold2
      thresMin = _ThresMin,
      thresMax = _ThresMax,
   }
end
 
local totalOdds = 0
local dynOddsTable = {
   { data = 0, curOdds = 0, configData = dotConfig( 99, 5, 1, 10) },
   { data = 1, curOdds = 0, configData = dotConfig( 90, 10, 1, 10) },
   { data = 2, curOdds = 0, configData = dotConfig( 80, 15, 1, 10) },
   { data = 5, curOdds = 0, configData = dotConfig( 30, 20, 1, 10) },
   { data = 10, curOdds = 0, configData = dotConfig( 25, 25, 1, 10) },
   { data = 15, curOdds = 0, configData = dotConfig( 20, 30, 1, 10) },
   { data = 20, curOdds = 0, configData = dotConfig( 15, 40, 1, 10) },
   { data = 25, curOdds = 0, configData = dotConfig( 10, 50, 1, 10) },
   { data = 30, curOdds = 0, configData = dotConfig( 5, 70, 1, 10) },
   { data = 40, curOdds = 0, configData = dotConfig( 4, 80, 1, 10) },
   { data = 70, curOdds = 0, configData = dotConfig( 2, 90, 1, 10) },
   { data = 100, curOdds = 0, configData = dotConfig( 1, 95, 1, 10) }
}
 
local function calcOdds( thres, config )
   local base = config.oddsMin
   local range = config.oddsMax - config.oddsMin
 
   -- the following is just a "linear interpolation" formula
   local ratio = 0
   if     thres <= config.thresMin then ratio = 0
   elseif thres >= config.thresMax then ratio = 1
   else   ratio = (thres-config.thresMin)/(config.thresMax-config.thresMin)
   end
 
   return base + (range*ratio)
end
 
local function updateTable( thres )
   totalOdds = 0
   for k,v in pairs(dynOddsTable) do
      v.curOdds = calcOdds(thres,v.configData)
      totalOdds = totalOdds + v.curOdds
   end
end

local function gimme()
   local diceroll = math.random(1,totalOdds)
   for k,v in pairs(dynOddsTable) do
      if diceroll <= v.curOdds then return v.data end
 
      diceroll = diceroll - v.curOdds
   end
end]]

local assets = {Asset("ANIM", "anim/papyrus.zip"), Asset("ATLAS", "images/inventoryimages/blacksmithkey.xml")}

local ChanceTable = {1,5,10,20,30,40,50,100,150,250,350,500,750,900,1000}

local function getstatus(inst)
	if GetClock():IsDay() or GetClock():IsDusk() and not GetWorld():IsCave() then
		return "DAY_NOREAD"
	elseif GetWorld():IsCave() then
		return "CAVE_NOREAD"
	else
		if inst.readable == true then
			return inst.blacksmithquote
		else
			return "ALREADY_READ_TONIGHT"
		end
	end
end

local function ExaminationDetected(inst, player)
	if not GetClock():IsDay() and not GetClock():IsDusk() and not GetWorld():IsCave() and inst.readable == true then
		if inst.timer <= 14 then inst.timer = inst.timer + 1 end
		if player.prefab == "markiplier" then
			if math.random(1,1000) <= ChanceTable[inst.timer] then
				inst:DoTaskInTime(2, function()
					player.ManiacNote(player, inst.timer)
					inst.timer = 1
				end)
			end
		end
		inst.blacksmithquote = "BLACKSMITH_"..tostring(inst.timer)
		inst:DoTaskInTime(0.1, function() inst.readable = false end)
	end
end

local function onsave(inst, data)
	data.blacksmithquote = inst.blacksmithquote
	data.timer = inst.timer
	data.readable = inst.readable
end

local function onload(inst, data)
	if data then
		inst.blacksmithquote = data.blacksmithquote or "BLACKSMITH_1"
		inst.timer = data.timer or 1
		inst.readable = data.readable or true
	end
end

local function fn(Sim)
    local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
    inst.AnimState:SetBank("papyrus")
    inst.AnimState:SetBuild("papyrus")
    inst.AnimState:PlayAnimation("idle")
	inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("blacksmithnote.tex")
	inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("markiplier")
	inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/blacksmithnote.xml"
	inst.blacksmithquote = "BLACKSMITH_1"
	inst.timer = 1
	inst.readable = true
	inst:ListenForEvent( "daycomplete", function()
		if inst.readable ~= true then inst.readable = true end
	end, GetWorld())
	inst:AddComponent("inspectable")
	inst.OnSave = onsave
	inst.OnLoad = onload
	inst.components.inspectable.getstatus = getstatus
	inst.components.inspectable.description = ExaminationDetected
    return inst
end

return Prefab( "common/inventory/blacksmithnote", fn, assets) 