local function MakeaSafe(data)
	require "prefabutil"

	local assets = {
		Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ),
		Asset("ANIM", "anim/blacksmithsafe.zip"),
		Asset("ANIM", "anim/ui_blacksmith_chest_3x4.zip"),
	}

	local starting_inventory = {
		"blacksmithsword_blueprint",
		"meat_dried",
		"meat_dried",
		"goldenpickaxe",
		"rope",
		"rope",
		"rope",
		"cutstone",
		"cutstone",
		"hammer",
		"bedroll_furry",
		"axe",
		"goldnugget",
		"goldnugget",
		"goldnugget",
		"blacksmithplatinum",
		"rocks",
		"rocks",
		"rocks",
		"rocks",
		"rocks",
		"rocks",
		"rocks",
		"minerhat"
	}

	local function onopen(inst) 
		inst.AnimState:PlayAnimation("idleopen") 
		inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
	end 

	local function onclose(inst)
		inst.AnimState:PlayAnimation("idleunlocked") 
		inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
	end 

	local slotpos = {}

	for y = 2.5, -0.5, -1 do
		for x = 0, 2 do
			table.insert(slotpos, Vector3(75*x-75*2+75, 75*y-75*2+75,0))
		end
	end

	local function UnlockWilf(inst)
		local player = GetPlayer()
		player.profile:UnlockCharacter("wilford")
		player.profile.dirty = true
		player.profile:Save()

		GetWorld():PushEvent("wilfordunlocked")
	end

	local function OnLock(inst)

		inst.entity:AddAnimState():PlayAnimation("idlelocked")
		
	end

	local function StockSafe(inst)
			for k,v in pairs(starting_inventory) do
				local NewItem = SpawnPrefab(v)
				if NewItem.components.inventoryitem then
				inst.components.container:GiveItem(NewItem)
			end
		end
	end

	local function OnUnlock(inst)

		inst.entity:AddAnimState():PlayAnimation("unlocking")
		inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/safeunlock")
		inst:ListenForEvent("animover", function()
			local NewSafe = SpawnPrefab("unlockedblacksmithsafe")
			NewSafe.Transform:SetPosition(inst:GetPosition():Get())
			StockSafe(NewSafe)
			SpawnPrefab("blacksmithkey").Transform:SetPosition(inst:GetPosition():Get())
			UnlockWilf()
			inst:Remove()
		end)

	end

	local function fn (Sim)
		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()
		
		inst.Transform:SetScale(2,2,2)
		
		inst:AddTag("blacksmithsafe")
		inst:AddTag("irreplaceable")
		
		anim:SetBank("blacksmithsafe")
		anim:SetBuild("blacksmithsafe")
		
		inst:AddComponent("characterspecific")
		inst.components.characterspecific:SetOwner("markiplier")
		
		inst:AddComponent("inspectable")
		
		inst.entity:AddMiniMapEntity()
		inst.MiniMapEntity:SetIcon("blacksmithsafe.tex")
		
		if data.state == "unlocked" then
			inst.entity:AddAnimState():PlayAnimation("idleunlocked")
			inst:AddComponent("container")
			inst.components.container:SetNumSlots(#slotpos)
			inst.components.container.widgetslotpos = slotpos
			inst.components.container.widgetanimbank = "ui_chester_shadow_3x4"
			inst.components.container.widgetanimbuild = "ui_blacksmith_chest_3x4"	
			inst.components.container.widgetpos = Vector3(0,200,0)
			inst.components.container.side_align_tip = 160
			inst.components.container.onopenfn = onopen
			inst.components.container.onclosefn = onclose
			inst:AddTag("blacksmithsafeunlocked")
		else
			inst:AddComponent("lock")
			inst.components.lock.locktype = "blacksmith"
			inst.components.lock:SetOnUnlockedFn(OnUnlock)
			inst.components.lock:SetOnLockedFn(OnLock)
			OnLock(inst)
		end
		
		MakeSnowCovered(inst, 1)
		
		return inst
	end
	return Prefab( "common/"..data.state.."blacksmithsafe", fn, assets, starting_inventory)
end

local data = {
	{state="locked"},
	{state="unlocked"}
}

local ret = {}
for k,v in pairs(data) do
	table.insert(ret, MakeaSafe(v))
end

return unpack(ret)