local assets = {
    Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ),
	Asset( "ANIM", "anim/blacksmithsword.zip"),
	Asset( "ANIM", "anim/swap_blacksmithsword.zip"),
}

local prefabs = {
	"bleed",
	"bubbles",
	"blacksmithportal"
}

local function getrandomposition(inst)
	local ground = GetWorld()
	local centers = {}
	for i,node in ipairs(ground.topology.nodes) do
		local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
		if tile and tile ~= GROUND.IMPASSABLE then
			table.insert(centers, {x = node.x, z = node.y})
		end
	end
	if #centers > 0 then
		local pos = centers[math.random(#centers)]
		return Point(pos.x, 0, pos.z)
	else
		return inst.components.inventoryitem.owner:GetPosition()
	end
end

local function onattack(inst, attacker, target)
	inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/swordslash")
	if not target.SoundEmitter then
		target.entity:AddSoundEmitter()
	end
	local Special = math.random(1,100)
	if Special <= 20 then
		local ChoosePower = math.random(1,4)
		if ChoosePower == 1 then
			if target.components.burnable and not target.components.burnable:IsBurning() then
				if target.components.freezable and target.components.freezable:IsFrozen() then           
					target.components.freezable:Unfreeze()            
				else            
					target.components.burnable:Ignite(true)
				end   
			end
		elseif ChoosePower == 2 then
			if target.components.freezable then
				target.components.freezable:AddColdness(10)
				target.components.freezable:SpawnShatterFX()
			end
			if target.components.burnable and target.components.burnable:IsBurning() then
				target.components.burnable:Extinguish()
			end
		elseif ChoosePower == 3 then
			if target.components.locomotor then
				local CurrentWalkSpeed = target.components.locomotor.walkspeed
				local CurrentRunSpeed = target.components.locomotor.runspeed
				target.components.locomotor.walkspeed = CurrentWalkSpeed *0.5
				target.components.locomotor.runspeed = CurrentRunSpeed *0.5
			end
			if target.components.combat then
				local CurrentAttackSpeed = target.components.combat.min_attack_period
				target.components.combat.min_attack_period = CurrentAttackSpeed *2
			end
			target.AnimState:SetMultColour(0.5,1,0.5,0.9)
			target.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/poisoned", "poisoned")
			target.poisoned = true
			target.PoisonTime = 10
			if target.bubbleTask == nil then
				target.bubbleTask = target:DoPeriodicTask(0.5, function()
				SpawnPrefab("bubbles").Transform:SetPosition(target:GetPosition():Get())
				target.PoisonTime = target.PoisonTime -0.5
					if target.PoisonTime <= 1 then
						target.bubbleTask:Cancel()
						target.bubbleTask = nil
					end
				end)
			end
			target:DoTaskInTime(10, function()
				if target.components.locomotor then
					local CurrentWalkSpeed = target.components.locomotor.walkspeed
					local CurrentRunSpeed = target.components.locomotor.runspeed
					target.components.locomotor.walkspeed = CurrentWalkSpeed *2
					target.components.locomotor.runspeed = CurrentRunSpeed *2
				end
				if target.components.combat then
					local CurrentAttackSpeed = target.components.combat.min_attack_period
					target.components.combat.min_attack_period = CurrentAttackSpeed *0.5
				end
				target.AnimState:SetMultColour(1,1,1,1)
				target.SoundEmitter:KillSound("poisoned")
				target.poisoned = false
			end)
		elseif ChoosePower == 4 then
			if target.components.health then
				target.AnimState:SetMultColour(1,0.5,0.5,1)
				SpawnPrefab("bleed").Transform:SetPosition(target:GetPosition():Get())
				target.bleedCount = math.random(5,9)
				if target.bleedTask == nil then
					target.bleedTask = target:DoPeriodicTask(1, function()
						target.AnimState:SetMultColour(1,0,0,1)
						SpawnPrefab("bleed").Transform:SetPosition(target:GetPosition():Get())
						target.components.health:DoDelta(-5)
						target:DoTaskInTime(0.2, function() target.AnimState:SetMultColour(1,0.5,0.5,1) end)
						target.bleedCount = target.bleedCount -1
						if target.bleedCount <= 1 then
							target:DoTaskInTime(0.3, function()
								if target.poisoned == true then
									target.AnimState:SetMultColour(0.5,1,0.5,0.9)
								else
									target.AnimState:SetMultColour(1,1,1,1)
								end
								target.bleedTask:Cancel()
								target.bleedTask = nil
							end)
						end
					end)
				end
			end
		end
	elseif Special >= 99 then
		local ChanceToWarp = math.random(1,100)
		if ChanceToWarp <= 50 then
			if not inst:HasTag("markimob") then
				if target.components.locomotor then
					target.components.locomotor:StopMoving()
				end
				SpawnPrefab("blacksmithportal").Transform:SetPosition(target:GetPosition():Get())
				target.SizeReduction = 1
				if target.Reduce == nil then
					target.components.health:SetInvincible(true)
					target.Reduce = target:DoPeriodicTask(0.1, function()
						target.SizeReduction = target.SizeReduction - 0.1
						target.Transform:SetScale(target.SizeReduction,target.SizeReduction,target.SizeReduction)
						if target.SizeReduction <= 0.1 then
							target.Reduce:Cancel()
							target.Reduce = nil
							local t_loc = getrandomposition(inst)
							target.Transform:SetPosition(t_loc.x, t_loc.y, t_loc.z)
							target.Transform:SetScale(1,1,1)
							target.components.health:SetInvincible(false)
						end
					end)
				end
			end
		end
	end
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_blacksmithsword", "swap_ruins_bat")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function onfinished(inst)
	local owner = inst.components.inventoryitem.owner
	local Platinum = SpawnPrefab("blacksmithplatinum")
	owner.components.inventory:GiveItem(Platinum)
	SpawnPrefab("brokentool").Transform:SetPosition(owner:GetPosition():Get())
	owner.AnimState:PlayAnimation("hit")
	owner.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
	inst:Remove()
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
   
	inst:AddTag("BLACKSMITHSWORD")
	inst:AddTag("sharp")
	inst:AddTag("irreplaceable")
    
    anim:SetBank("ruins_bat")
    anim:SetBuild("blacksmithsword")
    anim:PlayAnimation("idle")
	
    inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("markiplier")
	
    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(40)
    inst.components.weapon:SetOnAttack(onattack)
	
    inst:AddComponent("inspectable")
	
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/blacksmithsword.xml"
	
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )
	
	inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(20)
    inst.components.finiteuses:SetUses(200)
    inst.components.finiteuses:SetOnFinished(onfinished)
	
    return inst
end

return Prefab( "common/inventory/blacksmithsword", fn, assets) 