local assets = {
    Asset("ANIM", "anim/fedora.zip"),
    Asset("ATLAS", "images/inventoryimages/fedora.xml")
}

local function onequip(inst, owner) 
	owner.AnimState:OverrideSymbol("swap_hat", "fedora", "swap_hat")
	owner.AnimState:Show("HAT")
	owner.AnimState:Show("HAT_HAIR")
	owner.AnimState:Hide("HAIR_NOHAT")
	owner.AnimState:Hide("HAIR")
	if owner:HasTag("player") then
		owner.AnimState:Hide("HEAD")
		owner.AnimState:Show("HEAD_HAIR")
	end
	if owner.prefab == "markiplier" then
		if owner.MarkState ~= "Darkiplier" then
			owner.MarkState = "Fedoraplier"
			owner:ApplyFedora(owner)
		elseif owner.MarkState == "Darkiplier" then
			owner:DoTaskInTime(0.2, function(owner)
				owner:RemoveHat(owner)
				owner.components.talker:Say(STRINGS.MARKIMODSTRINGS.CANCEL_WILF_DARK)
			end)
		end
	else
		if ROG then
			inst.components.equippable.dapperness = TUNING.DAPPERNESS_LARGE
		else
			inst.components.dapperness.dapperness = TUNING.DAPPERNESS_LARGE
		end
	end
end

local function onunequip(inst, owner) 
	if owner.prefab == "markiplier" then
		if owner.MarkState == "Fedoraplier" then
			if owner.components.sanity.current >= owner.darkiplierthreshold then
				owner:BackToMark(owner)
			else
				owner:ApplyDarkness(owner)
			end
		end
	else
		if ROG then
			inst.components.equippable.dapperness = 0
		else
			inst.components.dapperness.dapperness = 0
		end
	end
	owner.AnimState:Hide("HAT")
	owner.AnimState:Hide("HAT_HAIR")
	owner.AnimState:Show("HAIR_NOHAT")
	owner.AnimState:Show("HAIR")
	if owner:HasTag("player") then
		owner.AnimState:Show("HEAD")
		owner.AnimState:Hide("HEAD_HAIR")
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
    
    inst:AddTag("hat")
	inst:AddTag("FEDORAPLIERHAT")
    
    anim:SetBank("rainhat")
    anim:SetBuild("fedora")
    anim:PlayAnimation("anim")
	
    inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("markiplier")
	
    inst:AddComponent("inspectable")
    
    inst:AddTag("irreplaceable")
    
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/fedora.xml"
    
    inst:AddComponent("equippable")
    inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
    
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )
	
	inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("fedora.tex")
	
	if ROG then
		inst:AddComponent("waterproofer")
		inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_SMALL * 0.5)
		inst.components.equippable.insulated = true
	else
		inst:AddComponent("dapperness")
	end
	
	inst.SpawnableFX = true
	
	inst:DoPeriodicTask(0.1, function()
		local owner = inst.components.inventoryitem.owner
		local FedFX = TheSim:FindFirstEntityWithTag("FedFX")
		if owner == nil then
			if not FedFX then
				if inst.SpawnableFX == true then
					SpawnPrefab("fedfx").Transform:SetPosition(inst:GetPosition():Get())
					inst.SpawnableFX = false
				end
			end
		else
			if FedFX then
				TheSim:FindFirstEntityWithTag("FedFX"):Remove()
				inst.SpawnableFX = true
			end
		end
	end)
    
    return inst
end

return Prefab( "common/inventory/fedora", fn, assets) 