require "prefabutil"
local SurgeryCompleteScreen = require "screens/surgerycompletescreen"

local assets = {
	Asset("SOUND", "sound/markiplier_sounds_bank00.fsb"),
	Asset("ANIM", "anim/patientbab.zip"),
	Asset("ANIM", "anim/ui_patientbab_1x1.zip")
}

local prefabs = {
	"fedora",
	"new_heart",
	"heartbox",
	"operatingtool_blueprint"
}

StateTable = { "one", "two", "three", "four", "five", "six" }

local function onopen(inst)
	inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/bloodsquirt")
end 

local function onclose(inst)
	inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/bloodsquirt")		
end 

local function onfinishoperate(inst, worker)
	inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/bloodsquirt")
	inst.State = 6
	inst.OperationState = 2
	inst:Update()
	inst:UpdateAnim()
end

local function onoperate(inst, worker, workleft)
	inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/bloodsquirt")
	if workleft ~= 1 and workleft ~= 5 and workleft ~= 10 and workleft ~= 15 and workleft ~= 20 then
		inst.AnimState:PlayAnimation("work_"..StateTable[inst.State])
	else
		inst.AnimState:PlayAnimation("work_"..StateTable[inst.State].."_organ")
	end
	if workleft == 19 then inst.State = 2 inst:UpdateAnim() end
	if workleft == 14 then inst.State = 3 inst:UpdateAnim() end
	if workleft == 9 then inst.State = 4 inst:UpdateAnim() end
	if workleft == 4 then inst.State = 5 inst:UpdateAnim() end
end

local slotpos = {Vector3(0,-(15+4),0)}

local function itemtest(inst, item, slot)
	if item.prefab ~= "new_heart" then
		return false
	else
		inst:DoTaskInTime( 0.2, function() if inst.OperationState ~= 3 then
			inst.OperationState = 3
			inst:Update()
			inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/bloodsquirt")
			TheFrontEnd:PushScreen(SurgeryCompleteScreen(inst))
		end end)
		return true
	end
end

local state = { "PREPPED", "BLEEDING", "FINISHED" }
local function getstatus(inst)
	return state[inst.OperationState]
end

local function StateOne(inst)
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.OPERATE)
	inst.components.workable:SetWorkLeft(25)
	inst.components.workable:SetOnFinishCallback(onfinishoperate)
	inst.components.workable:SetOnWorkCallback(onoperate) 
end

local function StateTwo(inst)
	if inst.components.workable then
		inst:RemoveComponent("workable")
	end
	inst:AddComponent("container")
	inst.components.container:SetNumSlots(#slotpos)
	inst.components.container.onopenfn = onopen
	inst.components.container.onclosefn = onclose
	inst.components.container.itemtestfn = itemtest
	inst.components.container.widgetslotpos = slotpos
	inst.components.container.widgetanimbank = "ui_cookpot_1x4"
	inst.components.container.widgetanimbuild = "ui_patientbab_1x1"
	inst.components.container.widgetpos = Vector3(0,200,0)
	inst.components.container.side_align_tip = 160
end

local function StateThree(inst)
	if inst.components.workable then
		inst:RemoveComponent("workable")
	end
	if inst.components.container then
		inst:RemoveComponent("container")
	end
end

local function Update(inst)
	if inst.OperationState == 1 then
		if not inst.components.workable then
			StateOne(inst)
		end
	elseif inst.OperationState == 2 then
		if not inst.components.container then
			StateTwo(inst)
		end
	elseif inst.OperationState == 3 then
		if inst.components.container then
			if inst.components.container:IsOpen() then
				inst.components.container:Close()
			end
			inst.components.container:DestroyContents()
		end
		StateThree(inst)
	end
end

local function UpdateAnim(inst)
	inst.AnimState:PlayAnimation("idle_"..StateTable[inst.State])
end

local function GiveHat(inst)
	local MLADIES = SpawnPrefab("fedora")
	if GetPlayer().prefab == "markiplier" then
		GetPlayer().components.inventory:GiveItem(MLADIES)
	end
end

local function onsave(inst, data)
	data.State = inst.State
	data.SpawnStuff = inst.SpawnStuff
	data.OperationState = inst.OperationState
end

local function onload(inst, data)
	inst.State = data.State or 1
	inst.SpawnStuff = data.SpawnStuff or true
	inst.OperationState = data.OperationState or 1
	inst:Update(inst)
	inst:UpdateAnim(inst)
end

local fn = function(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("patientbab.tex")
	inst.Transform:SetScale(2.5,2.5,2.5)
	
	inst.State = 1
	inst.OperationState = 1
	inst.SpawnStuff = true
	inst.OnSave = onsave
	inst.OnLoad = onload
	inst.Update = Update
	inst.UpdateAnim = UpdateAnim
	inst.GiveHat = GiveHat
	if inst.SpawnStuff == true and not TheSim:FindFirstEntityWithTag("donorbox") then
		inst:DoTaskInTime(0.5, function()
			local function TestPos(inst)
				local x,y,z = inst.Transform:GetWorldPosition()
				local function Randomize(inst)
					x,y,z = inst.Transform:GetWorldPosition()
					local Table1 = {}
					local Table2 = {math.random(3,7), math.random(3,7)}
					for i = 1,2 do
						if math.random(1,100) <= 50 then
							table.insert(Table1, (Table2[i])) else table.insert(Table1, -(Table2[i]))
						end
					end
					return Table1
				end
				local Position = nil
				local tile = nil
				repeat Position = Randomize(inst)
				x = x + Position[1] z = z + Position[2]
				tile = GetWorld().Map:GetTileAtPoint(x,y,z) until tile and tile ~= GROUND.IMPASSABLE
				return {x,y,z}
			end
			local boxPOS = TestPos(inst)
			local heartbox = SpawnPrefab("heartbox")
			heartbox.Transform:SetPosition(boxPOS[1], boxPOS[2], boxPOS[3])
			heartbox:Fill()
			local bpPOS = TestPos(inst)
			local blueprint = SpawnPrefab("operatingtool_blueprint")
			blueprint.Transform:SetPosition(bpPOS[1], bpPOS[2], bpPOS[3])
			inst.SpawnStuff = false
		end)
	end

	inst:AddTag("patientbab")
	inst.AnimState:SetBank("patientbab")
	inst.AnimState:SetBuild("patientbab")
	inst.AnimState:PlayAnimation("idle_"..StateTable[inst.State])
	inst:Update(inst)
	inst:UpdateAnim(inst)
	
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	
	MakeSnowCovered(inst, .01)	
	return inst
end

return Prefab( "common/patientbab", fn, assets, prefabs)