require "prefabutil"

local assets = {
	Asset("SOUND", "sound/markiplier_sounds_bank00.fsb"),
	Asset("ANIM", "anim/ui_cookpot_1x4.zip"),
    Asset("ANIM", "anim/tinyboxtim.zip"),
	Asset("ANIM", "anim/tinywarfboxtim.zip"),
	Asset("ANIM", "anim/euphoricboxtim.zip"),
	Asset("ANIM", "anim/euphoricwarfboxtim.zip")
}

local prefabs = { "tinytimdevice" }

local sweets = {}
local meats = {}
local veggies = {}

local function OnOpen(inst)
	inst.sg:GoToState("open")
	inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.on_open))
end 

local function OnClose(inst) 
    inst.sg:GoToState("close")
	inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH.on_close))
end 

local function itemtest(inst, item, slot)
	if item.prefab == "chester_eyebone" or item.prefab == "tinytimdevice" or item.prefab == "lexitoy" then
		return false
	else
		return true
	end
end

local function OnStartFollowing(inst) 
    inst:AddTag("companion") 
end

local function getstatus(inst)
	if inst.Warfified == true and inst.Euphoric == false then return "WARFBOX"
	elseif inst.Euphoric == true and inst.Warfified == false then return "EUPHORIC"
	elseif inst.Euphoric == true and inst.Warfified == true then return "WARPHORIC"
	elseif inst.Warfified == false and inst.Euphoric == false then return "GENERIC"
	end
end

local slotpos = {
	Vector3(0,64+32+8+15,0), 
	Vector3(0,32+18,0),
	Vector3(0,-(15+4),0), 
	Vector3(0,-(64+10+8+4),0),
	Vector3(0,-(64+28+8+55),0)
}

local function create_tinyboxtim()
    local inst = CreateEntity()
    
    inst:AddTag("companion")
    inst:AddTag("character")
    inst:AddTag("tinyboxtim")
    inst:AddTag("notraptrigger")

    inst.entity:AddTransform()

    inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("tinyboxtim.tex")
    inst.entity:AddAnimState()
    inst.AnimState:SetBank("tinyboxtim")
    inst.AnimState:SetBuild("tinyboxtim")
	inst.AnimState:PlayAnimation("idle")
	inst.Transform:SetScale(0.3,0.3,0.3)

    inst.entity:AddSoundEmitter()

    inst.entity:AddDynamicShadow()
    inst.DynamicShadow:SetSize( 1, 1 )

    MakeCharacterPhysics(inst, 75, .5)
    
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)

    inst.Transform:SetFourFaced()

    inst:AddTag("noauradamage")
	
    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 50
    inst.components.locomotor.runspeed = 50
	inst.components.locomotor.fasteronroad = true
	
	inst:AddComponent("talker")
	inst.components.talker.colour = Vector3(150/255,100/255,50/255)
	inst.components.talker.offset = Vector3(0,-500,0)
	
	local commonstringtable = STRINGS.MARKIMODSTRINGS.TINYBOXTIM_SPEECH
	inst:AddComponent("chatterbox")
	inst.components.chatterbox:SetCondition(function()
		if not inst.sg:HasStateTag("talking") and inst.components.chatterbox.run_convo == false then
			if not inst.sg:HasStateTag("sleeping") then return true
			else return false end
		else return false end
	end)
	inst.components.chatterbox:SetIdleChat(function()
		if inst.components.chatterbox:CheckSurroundings() ~= "cave" then
			if not inst.sg:HasStateTag("open") then
				if math.random(1,100) <= 50 then
					local Time = inst.components.chatterbox:CheckSurroundings()
					if commonstringtable[Time] then
						if Time == "fullmoon" then
							if math.random(1,100) <= 20 then
							inst.components.talker:Say(GetRandomItem(commonstringtable["night"])) else
							inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
						else inst.components.talker:Say(GetRandomItem(commonstringtable[Time])) end
					else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
				else inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"])) end
			else inst.components.talker:Say(GetRandomItem(commonstringtable["open"])) end
		else
			if math.random(1,100) <= 40 then inst.components.talker:Say(GetRandomItem(commonstringtable["idle_chat"]))
			else inst.components.talker:Say(GetRandomItem(commonstringtable["cave_chat"])) end
		end
	end)
	inst.components.chatterbox:SetItemTracker({
		giants = {"moose", "dragonfly", "deerclops", "bearger"},
		teleportatopieces = {"teleportato_base", "teleportato_crank", "teleportato_ring", "teleportato_potato", "teleportato_box"},
		pig_king = {"pigking", "pigking"},
		blacksmithsafe = {"lockedblacksmithsafe", "lockedblacksmithsafe"},
		patient = {"patientbab", "patientbab"}
	})
	inst.components.chatterbox:EnableConversing(50, 150, 70, 25, commonstringtable, "tim")
	inst.components.chatterbox:IdleChat(10, 25, 60)
	inst.components.chatterbox:Encourage(1, 5, 80, "markiplier", 20, commonstringtable["mark_working"])
	inst.components.chatterbox:LookForItems(10, 40, 65, 75, commonstringtable)

    inst:AddComponent("follower")
	
    inst:AddComponent("knownlocations")
    MakeSmallBurnableCharacter(inst, "chester_body")
	
    inst:AddComponent("container")
    inst.components.container:SetNumSlots(5)
	
    inst.components.container.onopenfn = OnOpen
    inst.components.container.onclosefn = OnClose
	inst.components.container.itemtestfn = itemtest
	
	inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("markiplier")
    
    inst.components.container.widgetslotpos = slotpos
    inst.components.container.widgetanimbank = "ui_cookpot_1x4"
    inst.components.container.widgetanimbuild = "ui_cookpot_1x4"
    inst.components.container.widgetpos = Vector3(0,200,0)
    inst.components.container.side_align_tip = 160

    inst:SetStateGraph("SGtinyboxtim")
    inst.sg:GoToState("idle")
	
	inst:DoTaskInTime(0.1, function(inst)
        if not TheSim:FindFirstEntityWithTag("tinytimdevice") then
            inst:Remove()
        end
    end)
	
	inst:AddComponent("sanityaura")
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	inst:AddComponent("named")
	inst.Warfified = false
	inst.Euphoric = false
	inst.TimState = "tinyboxtim"
	inst:DoPeriodicTask(0.1, function(inst)
		if not inst.sg:HasStateTag("sleeping") then 
			local WarfMe = false
			local FancyMe = false
			for k,v in pairs(inst.components.container.slots) do
				if v then
					if v.prefab == "warfmini" or v.prefab == "warfstache" or v.prefab == "warfstaff" or v.prefab == "warfbug" or v.prefab == "poofgun" or v.prefab == "warfbutter" then
						WarfMe = true
					end
					if v.prefab == "fedora" then
						FancyMe = true
					end
				end
			end
			local function ChangeTim(inst, name, stringname, sanity, speech_color)
				inst.AnimState:SetBank(name)
				inst.AnimState:SetBuild(name)
				inst.MiniMapEntity:SetIcon(name..".tex")
				inst.TimState = name
				inst.components.named:SetName(stringname)
				inst.components.sanityaura.aurafn = function() return TUNING.SANITYAURA_TINY * sanity end
				if type(speech_color) == "number" then
					inst.components.talker.colour = Vector3(speech_color/255,speech_color/255,speech_color/255)
				else
					inst.components.talker.colour = Vector3(speech_color[1]/255,speech_color[2]/255,speech_color[3]/255)
				end
				if inst.sg:HasStateTag("open") then inst.sg:GoToState("open") else inst.sg:GoToState("idle") end
			end
			if WarfMe == true and FancyMe == false and (inst.Warfified == false or (inst.Warfified == true and inst.Euphoric == true)) then
				if inst.Euphoric == true then inst.Euphoric = false end
				inst.Warfified = true
				ChangeTim(inst, "tinywarfboxtim", "Tiny WarfBox Tim", 2.5, {255, 161, 161})
			elseif FancyMe == true and WarfMe == false and (inst.Euphoric == false or (inst.Warfified == true and inst.Euphoric == true)) then
				if inst.Warfified == true then inst.Warfified = false end
				inst.Euphoric = true
				ChangeTim(inst, "euphoricboxtim", "Euphoric Box Tim", 3, 200)
			elseif WarfMe == true and FancyMe == true and (inst.Warfified == false or inst.Euphoric == false) then
				if inst.Warfified == false then inst.Warfified = true end
				if inst.Euphoric == false then inst.Euphoric = true end
				ChangeTim(inst, "euphoricwarfboxtim", "Euphoric WarfBox Tim", 5, {255, 200, 200})
			elseif WarfMe == false and FancyMe == false and (inst.Warfified == true or inst.Euphoric == true) then
				if inst.Warfified == true then inst.Warfified = false end
				if inst.Euphoric == true then inst.Euphoric = false end
				ChangeTim(inst, "tinyboxtim", "Tiny Box Tim", 0.5, {150, 100, 50})
			end
		end
	end)
	
	local brain = require "brains/tinyboxtimbrain"
    inst:SetBrain(brain)
	
    return inst
end

return Prefab( "common/tinyboxtim", create_tinyboxtim, assets, prefabs) 