require "prefabutil"

local WAKE_TO_FOLLOW_DISTANCE = 2
local SLEEP_NEAR_LEADER_DISTANCE = 2

local assets = {
	Asset("ANIM", "anim/ui_chest_3x3.zip"),
    Asset("ANIM", "anim/chester.zip")
}

local prefabs = {"lexitoy"}

local function ShouldWakeUp(inst)
    return DefaultWakeTest(inst) or not inst.components.follower:IsNearLeader(WAKE_TO_FOLLOW_DISTANCE)
end

local function ShouldSleep(inst)
    return DefaultSleepTest(inst) and not inst.sg:HasStateTag("open") and inst.components.follower:IsNearLeader(SLEEP_NEAR_LEADER_DISTANCE) and not GetWorld().components.clock:GetMoonPhase() == "full"
end


local function ShouldKeepTarget(inst, target)
    return false
end


local function OnOpen(inst)
        inst.sg:GoToState("open")
end 

local function OnClose(inst) 
        inst.sg:GoToState("close")
end 

local function itemtest(inst, item, slot)
	if item.prefab == "chester_eyebone" or item.prefab == "tinytimdevice" or item.prefab == "lexitoy" then
		return false
	else
		return true
	end
end

local function OnStartFollowing(inst) 
    inst:AddTag("companion") 
end

local slotpos = {}

for y = 2, 0, -1 do
    for x = 0, 2 do
        table.insert(slotpos, Vector3(80*x-80*2+80, 80*y-80*2+80,0))
    end
end

local function create_lexi()

    local inst = CreateEntity()
    
    inst:AddTag("companion")
    inst:AddTag("character")
    inst:AddTag("lexi")
    inst:AddTag("notraptrigger")

    inst.entity:AddTransform()

    inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("tinyboxtim.tex")
    inst.entity:AddAnimState()
    inst.AnimState:SetBank("chester")
    inst.AnimState:SetBuild("chester_build")
	inst.Transform:SetScale(0.8,0.8,0.8)

    inst.entity:AddSoundEmitter()

    inst.entity:AddDynamicShadow()
    inst.DynamicShadow:SetSize( 1, 1 )

    MakeCharacterPhysics(inst, 75, .5)
    
    inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    inst.Physics:CollidesWith(COLLISION.OBSTACLES)
    inst.Physics:CollidesWith(COLLISION.CHARACTERS)

    inst.Transform:SetFourFaced()

    inst:AddTag("noauradamage")
	
    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 7
    inst.components.locomotor.runspeed = 7
	inst.components.locomotor.fasteronroad = true

    inst:AddComponent("follower")
	
    inst:AddComponent("knownlocations")
    --MakeSmallBurnableCharacter(inst, "chester_body")
	
    inst:AddComponent("container")
    inst.components.container:SetNumSlots(9)
	
    inst.components.container.onopenfn = OnOpen
    inst.components.container.onclosefn = OnClose
	inst.components.container.itemtestfn = itemtest
	
	inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("muyskerm")
    
    inst.components.container.widgetslotpos = slotpos
    inst.components.container.widgetanimbank = "ui_chest_3x3"
    inst.components.container.widgetanimbuild = "ui_chest_3x3"
    inst.components.container.widgetpos = Vector3(0,200,0)
    inst.components.container.side_align_tip = 160

    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(3)
    inst.components.sleeper.testperiod = GetRandomWithVariance(6, 2)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWakeUp)

    inst:SetStateGraph("SGchester")
    inst.sg:GoToState("idle")
	
	inst:DoTaskInTime(0.1, function(inst)
        if not TheSim:FindFirstEntityWithTag("lexitoy") then
            inst:Remove()
        end
    end)
	
    inst:AddComponent("inspectable")
	
	local brain = require "brains/lexibrain"
    inst:SetBrain(brain)
	
    return inst
end

return Prefab( "common/lexi", create_lexi, assets, prefabs) 
