local assets = {
	Asset("ANIM", "anim/muyskermgun1.zip"),
	Asset("ANIM", "anim/swap_muyskermgun1.zip"), 
	Asset("ANIM", "anim/muyskermgun2.zip"),
	Asset("ANIM", "anim/swap_muyskermgun2.zip"), 
	Asset("ANIM", "anim/muyskermgun3.zip"),
	Asset("ANIM", "anim/swap_muyskermgun3.zip"), 
	Asset("ATLAS", "images/inventoryimages/muyskermpistol1.xml"),
	Asset("ATLAS", "images/inventoryimages/muyskermpistol2.xml"),
	Asset("ATLAS", "images/inventoryimages/muyskermpistol3.xml")
}

local scope_assets = {
	Asset("ANIM", "anim/scope.zip"),
	Asset("ATLAS", "images/inventoryimages/muyskerm_scope.xml")
}

local prefabs = {
    "muyskerm_projectile_1",
	"muyskerm_projectile_2",
	"muyskerm_projectile_3"
}

local scope_prefabs = {"muyskerm_pistol"}

local function MakeTalk(owner, subject)
	if owner and owner.components.talker then
		if owner.prefab == "markiplier" then
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject][(string.lower(owner.MarkState))])
		elseif STRINGS.MARKIMODSTRINGS[subject][owner.prefab] == nil then
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject]["default"])
		else
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject][owner.prefab])
		end
	end
end

local function HeadShot(inst, attacker, target)
	if target.components.health then
		if math.random(1,100) <= 5 then
			target.components.health:DoDelta(-500)
			MakeTalk(attacker, "HEADSHOT")
			--ToDo add sound and special HEADSHOT fx
		end
	end
end

local function Magicness(inst, attacker, target)
	if target.components.health then
		target.MagicTime = math.random(50,100)
		if target.MagicTask == nil then
			target.MagicTask = target:DoPeriodicTask(0.1, function()
				SpawnPrefab("muyskermsparkles").Transform:SetPosition(target:GetPosition():Get())
				target.components.health:DoDelta(-0.1)
				target.MagicTime = target.MagicTime - 1
				if target.MagicTime <= 1 then
					target.MagicTask:Cancel()
					target.MagicTask = nil
				end
			end)
		end
	end
	--ToDo add sparkly sound effects
end

local function Heal(inst, attacker, target)
	if target.components.health then
		target.components.health:DoDelta(math.random(10,30))
		--ToDo add healing fx and increase loyalty of healed pigs/bunnymen also force anything attacking to stop
	end
end

local function onattack(inst, attacker, target)
	local state = inst.components.statecycler.currentstate
	if attacker then
		if target then
			if attacker.WilfordState and attacker.WilfordState ~= "Mighty" then attacker.components.stache:DoDelta(5) end
			if state == 1 then HeadShot(inst, attacker, target)
			elseif state == 2 then Magicness(inst, attacker, target)
			elseif state == 3 then Heal(inst, attacker, target)
			end
		end
	end
end

local function OnDropped(inst)
    inst.AnimState:PlayAnimation("idle")
end

local onequip = function(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_muyskermgun"..inst.components.statecycler.currentstate, "swap_boomerang")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.components.combat.min_attack_period = owner.components.combat.min_attack_period * 2
end

local onunequip = function(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
	owner.components.combat.min_attack_period = owner.components.combat.min_attack_period / 2
end

local stringnames = {"Scoped Pistol", "Magic Pistol", "Healing Pistol"}
local damage = {45, 30, 0}

local function RebuildTile(inst, state)
    if inst.components.inventoryitem:IsHeld() then
        local owner = inst.components.inventoryitem.owner
        inst.components.inventoryitem:RemoveFromOwner(true)
		inst.components.inventoryitem.atlasname = "images/inventoryimages/muyskermpistol"..state..".xml"
		inst.components.inventoryitem:ChangeImageName("muyskermpistol"..state)
		inst.components.named:SetName(stringnames[state])
        if owner.components.container then
            owner.components.container:GiveItem(inst)
        elseif owner.components.inventory then
            owner.components.inventory:GiveItem(inst)
        end
    end
end

local function OnCycle(inst, state, statetable)
	inst.AnimState:SetBuild("muyskermgun"..state)
	RebuildTile(inst, state)
	inst:RemoveComponent("weapon")
	inst:AddComponent("weapon")
    inst.components.weapon:SetRange(30, 40)
	inst.components.weapon:SetDamage(damage[state])
	inst.components.weapon:SetOnAttack(onattack)
	inst.components.weapon:SetProjectile("muyskerm_projectile_"..state)
end

local function getstatus(inst)
	return (string.upper(inst.components.statecycler.states[inst.components.statecycler.currentstate]))
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
	
	inst.OnCycle = OnCycle
	inst:AddComponent("statecycler")
	inst.components.statecycler.states = {"normal","magic","healing"}
	
	anim:SetBank("boomerang")
    anim:SetBuild("muyskermgun1")
    anim:PlayAnimation("idle")
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	inst:AddComponent("named")
	inst.components.named:SetName("Scoped Pistol")
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/muyskermpistol1.xml"
	inst.components.inventoryitem:ChangeImageName("muyskermpistol1")
    inst.components.inventoryitem:SetOnDroppedFn(OnDropped)
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )

    inst:AddComponent("weapon")
    inst.components.weapon:SetRange(30, 40)
	inst.components.weapon:SetDamage(45)
	inst.components.weapon:SetOnAttack(onattack)
	inst.components.weapon:SetProjectile("muyskerm_projectile_1")
    return inst
end

local function scope_fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    anim:SetBank("scope")
    anim:SetBuild("scope")
    anim:PlayAnimation("idle")
	inst:AddComponent("inspectable")
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/muyskerm_scope.xml"
	return inst
end

local function MakeSomeAmmo(data, runthrough)
	local projectile_assets = {Asset("ANIM", "anim/warf_projectile.zip")}
	local ammo_assets = {Asset("ANIM", "anim/muyskermammo.zip"), Asset("ATLAS", "images/inventoryimages/muyskerm_ammo_"..data.state..".xml")}
	
	local function onthrowncommon(inst)
		local owner = inst.components.projectile.owner.components.inventoryitem.owner
		if owner then
			if owner.components.inventory then
				local ammo, num_found = owner.components.inventory:Has("muyskerm_ammo_"..data.state, 1)
				if ammo then
					owner.components.inventory:ConsumeByName("muyskerm_ammo_"..data.state, 1)
				else
					MakeTalk(owner, "OUT_OF_AMMO")
					inst:Remove()
				end
			end
		end
	end
	
	local function OnHit(inst)
		inst:Remove()
	end
	
	local function OnMiss(inst, owner)
		MakeTalk(owner.components.inventoryitem.owner, "I_MISSED")
		if math.random(1,100) <= 20 then SpawnPrefab("muyskerm_ammo_"..data.state).Transform:SetPosition(inst:GetPosition():Get()) end
		inst:Remove()
	end
	
	local function projectile_fn(Sim)
		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()
		local sound = inst.entity:AddSoundEmitter()
		inst.Transform:SetFourFaced()
		
		MakeInventoryPhysics(inst)
		RemovePhysicsColliders(inst)
		anim:SetBank("warf_projectile")
		anim:SetBuild("warf_projectile")
		inst:AddTag("projectile")
		inst:AddComponent("projectile")
		inst.components.projectile:SetOnHitFn(OnHit)
		inst.components.projectile:SetOnMissFn(OnMiss)
		inst:ListenForEvent("onthrown", onthrowncommon)
		inst.components.projectile:SetCanCatch(false)
		inst.AnimState:PlayAnimation(data.name.."_muygun", true)
		inst.Transform:SetScale(1.2, 1.2, 1.2)
		inst.components.projectile:SetSpeed(15)
		inst.components.projectile:SetHoming(false)
		inst.components.projectile.range = 30
		if data.light then
			inst.entity:AddLight()
			inst.Light:SetRadius(2)
			inst.Light:SetFalloff(.6)
			inst.Light:SetIntensity(.5)
			inst.Light:SetColour(200/255,20/255,200/255)
			inst.Light:Enable(true)
		end
		return inst
	end
	
	local function ammo_fn(Sim)
		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()
		anim:SetBank("muyskermammo")
		anim:SetBuild("muyskermammo")
		anim:PlayAnimation("idle_"..data.name)
		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = 99
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.atlasname = "images/inventoryimages/muyskerm_ammo_"..data.state..".xml"
		return inst
	end

	if runthrough == 1 then
		return  Prefab( "common/inventory/muyskerm_projectile_"..data.state, projectile_fn, projectile_assets )
	else
		return 	Prefab( "common/inventory/muyskerm_ammo_"..data.state, ammo_fn, ammo_assets )
	end
end

local data = {
	{name="normal", state=1, light=false},
	{name="magic", state=2, light=true},
	{name="healing", state=3, light=false}
}

local ret = {}
for k,v in pairs(data) do
	table.insert(ret, MakeSomeAmmo(v,1))
	table.insert(ret, MakeSomeAmmo(v,2))
end

return Prefab( "common/inventory/muyskerm_pistol", fn, assets, prefabs ),
	   Prefab( "common/inventory/muyskerm_scope", scope_fn, scope_assets, scope_prefabs ),
	   unpack(ret)