local function MakeFX(data)
	local temp_assets = {}
	if data.name ~= "fedfx" then
		temp_assets = {Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ), Asset( "ANIM", "anim/"..data.name..".zip" )}
	else
		temp_assets = {Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ), Asset( "ANIM", "anim/warfx.zip" )}
	end
	local assets = temp_assets
	local function fn (Sim)
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()

		if data.name == "fedfx" then
			inst.AnimState:SetBank("warfx")
			inst.AnimState:SetBuild("warfx")
		else
			inst.AnimState:SetBank(data.name)
			inst.AnimState:SetBuild(data.name)
		end
		inst:AddTag("FX")
		inst:AddTag("NOCLICK")
		if data.name == "warfx" or data.name == "fedfx" then
			inst.AnimState:PlayAnimation("sparkle", true)
		elseif data.name == "muyskermsparkles" then
			inst.AnimState:PlayAnimation("sparkle_"..(tostring(math.random(1,6))))
		else
			inst.AnimState:PlayAnimation(data.name)
		end
		if data.name == "warfx" or data.name == "fedfx" then
			inst:DoTaskInTime(0.1, function()
				local WarFXSize = MarkMod.WarFXSize
				inst.Transform:SetScale(WarFXSize, WarFXSize, WarFXSize)
			end)
			inst:DoPeriodicTask(0.1, function()
				local warfstache = TheSim:FindFirstEntityWithTag("warfstache")
				local fedora = TheSim:FindFirstEntityWithTag("FEDORAPLIERHAT")
				if warfstache then
					if data.name == "warfx" then inst.Transform:SetPosition(warfstache:GetPosition():Get()) end
				end
				if fedora then
					if data.name == "fedfx" then inst.Transform:SetPosition(fedora:GetPosition():Get()) end
				end
			end)
			if data.name == "warfx" then inst:AddTag("WarFX") elseif data.name == "fedfx" then inst:AddTag("FedFX") end
			inst.persists = false
		else
			if data.sound ~= nil then
				inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/fx/"..data.sound, data.name)
				inst.SoundEmitter:SetVolume(data.name, data.volume)
			end
			if data.scale ~= nil then
				inst.Transform:SetScale(data.scale,data.scale,data.scale)
			end

			inst:AddTag(data.name)
			if data.name == "shirtrip" then
				inst.FadeTime = 1
				inst:ListenForEvent("animover", function()
					if inst:HasTag("shirtrip") then
						inst:RemoveTag("shirtrip")
						inst.AnimState:PlayAnimation("idle")
						inst:DoPeriodicTask(0.1, function()
							inst.FadeTime = inst.FadeTime - 0.05
							inst.Transform:SetScale(inst.FadeTime,inst.FadeTime,inst.FadeTime)
							if inst.FadeTime <= 0 then
								inst:Remove()
							end
						end)
					end
				end)
			else
				inst:ListenForEvent("animover", function()
					inst:Remove()
				end)
			end
		end
		if data.light then
			inst.entity:AddLight()
			inst.Light:SetRadius(2)
			inst.Light:SetFalloff(.6)
			inst.Light:SetIntensity(.5)
			inst.Light:SetColour(200/255,20/255,200/255)
			inst.Light:Enable(true)
		end
		return inst
	end
	return Prefab( "common/"..data.name, fn, assets)
end

local data = {
	{name="warfx", 			  sound=nil,			volume=nil, scale=nil},
	{name="fedfx", 			  sound=nil,			volume=nil, scale=nil},
	{name="blacksmithportal", sound="blackhole",	volume=0.8, scale=1.5},
	{name="bleed", 			  sound="bloodsquirt",	volume=1.0, scale=0.2},
	{name="bubbles", 		  sound=nil,			volume=nil, scale=1.5},
	{name="poofcloud", 		  sound="poof",			volume=0.1, scale=0.7},
	{name="shirtrip", 		  sound="rip",			volume=0.9, scale=nil},
	{name="muyskermsparkles", sound=nil,			volume=nil, scale=1.5, light=true}
}

local ret = {}
for k,v in pairs(data) do
	table.insert(ret, MakeFX(v))
end

return unpack(ret)