local assets = {
	Asset("ANIM", "anim/booze.zip"),
	Asset("ANIM", "anim/groundbooze.zip"),
	Asset("ATLAS", "images/inventoryimages/booze.xml"),
}

local function getrandomposition(inst)
    local ground = GetWorld()
    local centers = {}
    for i,node in ipairs(ground.topology.nodes) do
        local tile = GetWorld().Map:GetTileAtPoint(node.x, 0, node.y)
        if tile and tile ~= GROUND.IMPASSABLE then
            table.insert(centers, {x = node.x, z = node.y})
        end
    end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return Point(pos.x, 0, pos.z)
    else
        return inst:GetPosition()
    end
end

local function drink(inst, eater)
	if eater then
		local Talker = eater.prefab
		if STRINGS.MARKIMODSTRINGS.DRINK[Talker] == nil then
			Talker = "default"
		end
		TheCamera:SetDistance(12)
		eater.AnimState:PlayAnimation("horn")
		eater.AnimState:OverrideSymbol("horn01", "booze", "horn01")
		eater.components.locomotor:StopMoving()
		eater.components.playercontroller:Enable(false)
		eater.components.health:SetInvincible(true)
		eater.Physics:Stop()
		eater:DoTaskInTime(2, function()
			SpawnPrefab("thrownbooze").Transform:SetPosition(eater:GetPosition():Get())
			TheFrontEnd:Fade(false,2)
			if eater.components.talker then
				eater.components.talker:Say(STRINGS.MARKIMODSTRINGS.DRINK[Talker])
			end
			eater.sg:GoToState("death")
		end)
		eater:DoTaskInTime(5, function()
			eater.sg:GoToState("sleep")
			eater.components.hunger:DoDelta(-20)
			eater.components.health:DoDelta(-3)
			eater.components.sanity:DoDelta(-75)
			local tlprt = getrandomposition(inst)
			eater.Transform:SetPosition(tlprt.x, tlprt.y, tlprt.z)
			GetClock():NextPhase()
		end)
		eater:DoTaskInTime(7, function()
			TheFrontEnd:Fade(true,1)
			eater.components.talker:Say(STRINGS.MARKIMODSTRINGS.AWAKE[Talker])
			eater.components.playercontroller:Enable(true)
			eater.components.health:SetInvincible(false)
			TheCamera:SetDistance(30)
		end)
		eater:DoTaskInTime(8.5, function()
		eater.sg:GoToState("wakeup")
		end)
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	local anim = inst.entity:AddAnimState()
    MakeInventoryPhysics(inst)
	
	inst:AddTag("drink")
    
    anim:SetBank("groundbooze")
    anim:SetBuild("groundbooze")
    anim:PlayAnimation("idle")
    
    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = 5

    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.atlasname = "images/inventoryimages/booze.xml"
	
	inst:AddComponent("edible")
	
	inst.components.edible:SetOnEatenFn(drink)
    
    return inst
end

return Prefab( "common/inventory/booze", fn, assets)