local MakePlayerCharacter = require "prefabs/player_common"
local MarkimodHelpScreen = require "screens/markimodcallforhelpscreen"

local assets = {
	Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ),
	Asset( "ANIM", "anim/santiplier_beard.zip" ),
	Asset( "ANIM", "anim/santiplier.zip" ),
}

local prefabs = {
	"booze",
	"pinkfuzz",
	"winterhat",
	"trunkvest_winter",
	"krampus_sack"
}

local start_inv = {
	"booze",
	"booze",
	"booze"
}

local santisack_inv = {
	"trinket_1",
	"trinket_2",
	"trinket_3",
	"trinket_4",
	"trinket_5",
	"trinket_6",
	"trinket_7",
	"trinket_8",
	"trinket_9",
	"trinket_10",
	"trinket_11",
	"trinket_12",
	"bluegem",
	"booze"
}

local function MakeMeDrunk(inst)
	inst:DoTaskInTime(math.random(10,30), function()
		local sanity = TUNING.DAPPERNESS_MED
		local multiplier = math.random(1,10)
		if math.random(1,100) <= 50 then
			inst.components.sanity.dapperness = sanity*multiplier
		else
			inst.components.sanity.dapperness = -sanity*multiplier
		end
		MakeMeDrunk(inst)
	end)
end

local function onsave(inst, data)
	data.seasonstart = inst.seasonstart
	data.worldstart = inst.worldstart
	data.HelpShown = inst.HelpShown
end

local function onload(inst, data)
	if data then
		inst.seasonstart = data.seasonstart or false
		inst.worldstart = data.worldstart or false
		MakeMeDrunk(inst)
		inst.HelpShown = data.HelpShown or false
		if inst.HelpShown == false then
			inst:DoTaskInTime(1, function() TheFrontEnd:PushScreen(MarkimodHelpScreen()) end)
		end
	end
end

local fn = function(inst)
	
	inst.soundsname = "santiplier"
	inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("santiplier.tex")
	inst:SetStateGraph("SGwilson")
	inst.AnimState:SetBuild("santiplier")
	inst:AddTag("santiplier")
	
	inst.HelpShown = false
	inst.seasonstart = false
	inst.worldstart = false
	inst.OnSave = onsave
	inst.OnLoad = onload
	
	inst:AddComponent("beard")
	
	if inst.seasonstart ~= true then
		GetWorld().components.seasonmanager:StartWinter()
		GetWorld().components.seasonmanager:Advance()
		GetWorld().components.seasonmanager:Advance()
		GetWorld().components.seasonmanager:ForcePrecip()
		GetWorld().components.seasonmanager.ground_snow_level = 5
		inst.seasonstart = true
	end
	inst:DoTaskInTime(0.1, function()
		if inst.worldstart ~= true then
			inst.AnimState:OverrideSymbol("beard", "santiplier_beard", "beard_medium")
			inst.components.beard.bits = 10
			inst.components.beard.daysgrowth = 1
			local hat = SpawnPrefab("winterhat")
			local coat = SpawnPrefab("trunkvest_winter")
			local hslot = hat.components.equippable.equipslot
			hat.components.inventoryitem:OnPutInInventory(inst)
			hat.components.equippable:Equip(inst)
			inst.components.inventory.equipslots[hslot] = hat
			local cslot = coat.components.equippable.equipslot
			coat.components.inventoryitem:OnPutInInventory(inst)
			coat.components.equippable:Equip(inst)
			inst.components.inventory.equipslots[cslot] = coat
			
			local theta = math.random() * 2 * PI
			local pt = inst:GetPosition()
			local radius1 = math.random(5, 7)
			local radius2 = math.random(10, 15)
			local offset1 = FindWalkableOffset(pt, theta, radius1, 12, true)
			local offset2 = FindWalkableOffset(pt, theta, radius2, 12, true)
			if offset1 then
				local santasack = SpawnPrefab("krampus_sack")
				local pos = pt + offset1
				santasack.Transform:SetPosition(pos:Get())
				for k,v in pairs(santisack_inv) do
					local NewItem = SpawnPrefab(v)
					santasack.components.container:GiveItem(NewItem)
				end
			end
			inst.worldstart = true
		end
	end)
	inst.components.beard.prize = "pinkfuzz"
	
    inst.components.beard.onreset = function() 
		inst.AnimState:ClearOverrideSymbol("beard")
	end
	
    inst.components.beard:AddCallback(1, function()
		inst.components.beard.bits = 10
		inst.AnimState:OverrideSymbol("beard", "beard", "beard_medium")
        if inst.components.beard.bits == 0 then
            inst.hairGrowthPending = true
			inst.AnimState:ClearOverrideSymbol("beard")
        end
    end)
	MakeMeDrunk(inst)
	
    inst.components.health:SetMaxHealth(150)
	inst.components.hunger:SetMax(200)
	inst.components.sanity:SetMax(125)
	inst.components.combat.damagemultiplier = 1
	inst.components.locomotor.walkspeed = (TUNING.WILSON_WALK_SPEED * 0.8)
	inst.components.locomotor.runspeed = (TUNING.WILSON_RUN_SPEED * 0.8)
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * 1.5)
	inst.components.sanity.neg_aura_mult = 0.5
    inst.components.sanity.night_drain_mult = 0
	inst.components.temperature:SetTemp( 25 )
    local Temperature = Class(function(self, inst)
		self.inherentinsulation = 25
    end)
end

return MakePlayerCharacter("santiplier", prefabs, assets, fn , start_inv)