local assets = {
	Asset("ANIM", "anim/poofgun.zip"),
	Asset("ANIM", "anim/swap_poofgun.zip"), 
	Asset("ATLAS", "images/inventoryimages/poofgun.xml")
}

local projectile_assets = {}

local prefabs = {
    "warf_projectile"
}

local function MakeTalk(owner, subject)
	if owner and owner.components.talker then
		if owner.prefab == "markiplier" then
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject][(string.lower(owner.MarkState))])
		elseif STRINGS.MARKIMODSTRINGS[subject][owner.prefab] == nil then
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject]["default"])
		else
			owner.components.talker:Say(STRINGS.MARKIMODSTRINGS[subject][owner.prefab])
		end
	end
end

local function onattack(inst, attacker, target)
	if attacker then
		local Buffed = attacker.WilfordState
		if target then
			MakeTalk(attacker, "POOF")
			if Buffed and Buffed ~= "Mighty" then attacker.components.stache:DoDelta(5) end
			SpawnPrefab("poofcloud").Transform:SetPosition(target:GetPosition():Get())
		end
	end
end

local function OnDropped(inst)
    inst.AnimState:PlayAnimation("idle")
end

local onequip = function(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_poofgun", "swap_boomerang")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
	owner.components.combat.min_attack_period = owner.components.combat.min_attack_period * 2
	if owner.prefab == "wilford" and owner.components.stache then
		owner.StacheBooster = 2
	end
end

local onunequip = function(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
	owner.components.combat.min_attack_period = owner.components.combat.min_attack_period / 2
	if owner.prefab == "wilford" and owner.components.stache and owner.StacheBooster == 2 then
		owner.StacheBooster = 0
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    
    anim:SetBank("boomerang")
    anim:SetBuild("poofgun")
    anim:PlayAnimation("idle")

    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/poofgun.xml"
    inst.components.inventoryitem:SetOnDroppedFn(OnDropped)
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )

    inst:AddComponent("weapon")
    inst.components.weapon:SetRange(20, 25)
	inst.components.weapon:SetDamage(30)
	inst.components.weapon:SetOnAttack(onattack)
	inst.components.weapon:SetProjectile("warf_projectile")

    return inst
end

local function OnHit(inst)
	inst:Remove()
end

local function OnMiss(inst, owner)
	MakeTalk(owner.components.inventoryitem.owner, "I_MISSED")
	SpawnPrefab("warfmini").Transform:SetPosition(inst:GetPosition():Get())
	inst:Remove()
end

local function onthrown(inst)
	local owner = inst.components.projectile.owner.components.inventoryitem.owner
	if owner then
		if owner.components.inventory then
			local ammo, num_found = owner.components.inventory:Has("warfmini", 1)
			if ammo then owner.components.inventory:ConsumeByName("warfmini", 1) else
				MakeTalk(owner, "OUT_OF_AMMO")
				inst:Remove()
			end
		end
	end
end

local function projectile_fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
	
	MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)
    anim:SetBank("warf_projectile")
    anim:SetBuild("warf_projectile")
    inst:AddTag("projectile")
    inst:AddComponent("projectile")
    inst.components.projectile:SetOnHitFn(OnHit)
	inst.components.projectile:SetOnMissFn(OnMiss)
	inst:ListenForEvent("onthrown", onthrown)
    inst.components.projectile:SetCanCatch(false)
    inst.AnimState:PlayAnimation("warf", true)
    inst.Transform:SetScale(1.2, 1.2, 1.2)
    inst.components.projectile:SetSpeed(15)
	inst.components.projectile:SetHoming(false)
	inst.components.projectile.range = 30
	
    return inst
end

return Prefab( "common/inventory/poofgun", fn, assets, prefabs),
	   Prefab( "common/inventory/warf_projectile", projectile_fn, projectile_assets)