require "brains/butterflybrain"
require "stategraphs/SGbutterfly"

local assets = {
	Asset("ANIM", "anim/warfbug.zip"),
	Asset("ATLAS", "images/inventoryimages/warfbug.xml")
}
    
local prefabs = {
    "warfbug_dead",
    "warfbutter",
    "warflower",
}

local function OnDropped(inst)
    inst.sg:GoToState("idle")
    if inst.components.workable then
        inst.components.workable:SetWorkLeft(1)
    end
	if inst.components.stackable then
	    while inst.components.stackable:StackSize() > 1 do
	        local item = inst.components.stackable:Get()
	        if item then
	            if item.components.inventoryitem then
	                item.components.inventoryitem:OnDropped()
	            end
	            item.Physics:Teleport(inst.Transform:GetWorldPosition() )
	        end
	    end
	end
end

local function OnPickedUp(inst)
end

local function OnWorked(inst, worker)
    if worker.components.inventory then
        worker.components.inventory:GiveItem(inst, nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
        worker.SoundEmitter:PlaySound("dontstarve/common/butterfly_trap")
    end
end


local function CanDeploy(inst) return true end

local function OnDeploy (inst, pt) 
    local warflower = SpawnPrefab("warflower")
    if warflower then
        warflower:PushEvent("growfrombutterfly")
		warflower.Transform:SetPosition(pt.x, pt.y, pt.z)
        inst.components.stackable:Get():Remove()
    end
end

local function GoHomeAction(inst)
    local flower = GetClosestInstWithTag("flower", inst, 100)
    if flower and 
       flower:IsValid() then
        return BufferedAction(inst, flower, ACTIONS.GOHOME, nil, Vector3(flower.Transform:GetWorldPosition() ))
    end
end

local function SpawnWarfFlowers(inst)
	inst.SpawningFlowers = true
	inst:DoTaskInTime(math.random(40,70), function()
		local A_total = 0
		local B_total = 0
		local dont_spawn = false
		local pos = inst:GetPosition()
		for k,v in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, 20)) do
			if v.prefab == "warflower" then
				A_total = A_total + 1
			elseif v.prefab == "warfbug" then
				B_total = B_total + 1
			end
			if inst.components.inventoryitem.owner then
				dont_spawn = true
			end
		end
		for i,e in pairs(TheSim:FindEntities(pos.x, pos.y, pos.z, 2)) do
			if e ~= inst then
				dont_spawn = true
				if e.prefab == "warflower" then
					if not inst.components.inventoryitem.owner then
						if GetClock():IsNight() or GetClock():IsDusk() then
							inst:Remove()
						end
					end
				end
			end
		end
		if dont_spawn == false then
			local Spawner = math.random(1,100)
			local SpawnChance = 50 - ((A_total + B_total) * 2.5)
			if Spawner <= SpawnChance then
				if GetClock():IsDay() then
					SpawnPrefab("warflower").Transform:SetPosition(inst:GetPosition():Get())
				end
			end
		end
		SpawnWarfFlowers(inst)
	end)
end

local function onsave(inst, data)
end

local function onload(inst, data)
    if data and data.anim then
        inst.animname = data.anim
	    inst.AnimState:PlayAnimation(inst.animname)
	end
	if inst.SpawningBugs == false then
		SpawnWarfbugs(inst)
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	
    inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    inst.Transform:SetTwoFaced()
	inst.entity:AddDynamicShadow()
	inst.DynamicShadow:SetSize( .8, .5 )
	
	inst:ListenForEvent( "dusktime", function() GoHomeAction(inst) end, GetWorld() )
    
    ----------
    
    inst:AddTag("butterfly")
    inst:AddTag("insect")
    inst:AddTag("smallcreature")
	inst:AddTag("warfbug")
   
    MakeCharacterPhysics(inst, 1, .25)
    inst.Physics:SetCollisionGroup(COLLISION.FLYERS)
    inst.Physics:ClearCollisionMask()
    inst.Physics:CollidesWith(COLLISION.WORLD)
    
    
    inst.AnimState:SetBuild("warfbug")
    inst.AnimState:SetBank("butterfly")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetRayTestOnBB(true);
    
    inst:AddComponent("locomotor")
    inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
    inst:SetStateGraph("SGbutterfly")
    
    ---------------------       
    inst:AddComponent("inventoryitem")
    inst:AddComponent("stackable")
    inst.components.inventoryitem:SetOnDroppedFn(OnDropped)
    inst.components.inventoryitem:SetOnPickupFn(OnPickedUp)
	inst.components.inventoryitem.canbepickedup = false
	inst.components.inventoryitem.nobounce = true
	inst.components.inventoryitem.atlasname = "images/inventoryimages/warfbug.xml"
    
    ------------------
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(1)


	inst:AddComponent("pollinator")
    ------------------
    
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "butterfly_body"
    ------------------

    inst:AddComponent("knownlocations")

    ------------------
    MakeSmallBurnableCharacter(inst, "butterfly_body")
    MakeTinyFreezableCharacter(inst, "butterfly_body")
    
    inst:AddComponent("inspectable")
    ------------------
    inst:AddComponent("lootdropper")
    inst.components.lootdropper:AddRandomLoot("warfbutter", 0.1)
    inst.components.lootdropper:AddRandomLoot("warfbug_dead", 5)   
    inst.components.lootdropper.numrandomloot = 1
    ------------------
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.NET)
    inst.components.workable:SetWorkLeft(1)
    inst.components.workable:SetOnFinishCallback(OnWorked)
    ------------------
    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = OnDeploy

    local brain = require "brains/butterflybrain"
    inst:SetBrain(brain)
	
	inst.SpawningFlowers = false
	SpawnWarfFlowers(inst)
	
	inst.OnSave = onsave 
    inst.OnLoad = onload 

    return inst
end

return Prefab( "forest/common/warfbug", fn, assets, prefabs),
       MakePlacer("common/warfbug_placer", "warflowers", "warflowers", "f1" ) 
