local function WarfStaffs(data)
	local assets = {
		Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ),
		Asset( "ANIM", "anim/"..data.build..".zip"),
		Asset( "ANIM", "anim/swap_"..data.build..".zip"),
		Asset( "ATLAS", "images/inventoryimages/"..data.name..".xml")
	}

	local function onequip(inst, owner) 
		owner.AnimState:OverrideSymbol("swap_object", "swap_"..data.build, "swap_hammer")
		owner.AnimState:Show("ARM_carry") 
		owner.AnimState:Hide("ARM_normal") 
		if data.name == "warfstaff" then
			if owner.prefab == "wilford" and owner.components.stache then
				owner.StacheBooster = 1
			end
		end
	end

	local function onunequip(inst, owner) 
		owner.AnimState:Hide("ARM_carry") 
		owner.AnimState:Show("ARM_normal")
		if data.name == "warfstaff" then
			if owner.prefab == "wilford" and owner.components.stache and owner.StacheBooster == 1 then
				owner.StacheBooster = 0
			end
		end
	end
	
	local function onfinished(inst)
		local owner = inst.components.inventoryitem.owner
		local EmptyStaff = SpawnPrefab("warfstaff_empty")
		owner.components.inventory:GiveItem(EmptyStaff)
		SpawnPrefab("brokentool").Transform:SetPosition(owner:GetPosition():Get())
		owner.AnimState:PlayAnimation("hit")
		owner.SoundEmitter:PlaySound("dontstarve/wilson/use_break")
		if owner.StacheBooster and owner.StacheBooster == 1 then
			owner.StacheBooster = 0
		end
		inst:Remove()
	end

	local function fn(Sim)
		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()
		MakeInventoryPhysics(inst)
	   
		inst:AddTag(data.name)
		inst:AddTag("sharp")
		inst:AddTag("irreplaceable")
		
		anim:SetBank("ham_bat")
		anim:SetBuild(data.build)
		anim:PlayAnimation("idle")
		
		inst:AddComponent("characterspecific")
		inst.components.characterspecific:SetOwner("wilford")
		
		inst:AddComponent("weapon")
		inst.components.weapon:SetDamage(data.damage)
		
		inst:AddComponent("inspectable")
		
		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.atlasname = "images/inventoryimages/"..data.name..".xml"
		
		inst:AddComponent("equippable")
		inst.components.equippable:SetOnEquip( onequip )
		inst.components.equippable:SetOnUnequip( onunequip )
		
		if data.name == "warfstaff" then
			inst:AddComponent("finiteuses")
			inst.components.finiteuses:SetMaxUses(100)
			inst.components.finiteuses:SetUses(100)
			inst.components.finiteuses:SetOnFinished(onfinished)
		end
		
		return inst
	end
	return Prefab( "common/inventory/"..data.name, fn, assets) 
end

local data =
			{
			{name="warfstaff", 		 build="warfstaff", 	  damage=40},
			{name="warfstaff_empty", build="empty_warfstaff", damage=1 }
			}

local ret = {}
for k,v in pairs(data) do
	table.insert(ret, WarfStaffs(v))
end

return unpack(ret)