local MakePlayerCharacter = require "prefabs/player_common"
local MarkimodHelpScreen = require "screens/markimodcallforhelpscreen"
local SwearControl = MarkMod.SwearControl

local assets = {
	Asset( "SOUND", "sound/markiplier_sounds_bank00.fsb" ),
	Asset( "ANIM", "anim/wilford.zip" ),
	Asset( "ANIM", "anim/mightywilford.zip" ),
	Asset( "IMAGE", "minimap/wilford.tex" ),
	Asset( "ATLAS", "minimap/wilford.xml" )
}

local prefabs = {
	"warfstaff",
	"warfstaff_empty",
	"warfbug",
	"warflower",
}

local start_inv = {
	"warfstaff",
	"poofgun_broken",
	"warfbug",
	"warfbug",
	"meat",
	"meat"
}

local function MightyWilford(inst)
	inst.WilfordState = "Mighty"
	inst.components.health:StartRegen(1, 5)
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * 6)
	inst.components.sanity.neg_aura_mult = (0.6)
    inst.components.sanity.night_drain_mult = (0.6)
	inst.components.health.absorb = (0.4)
	inst.components.combat.damagemultiplier = (1.9)
	inst.components.locomotor.walkspeed = (TUNING.WILSON_WALK_SPEED * 1.2)
	inst.components.locomotor.runspeed = (TUNING.WILSON_RUN_SPEED * 1.2)
	inst.Transform:SetScale(1.15,1.15,1.15)
	inst.components.eater:SetCarnivore()
	inst.AnimState:SetBuild("mightywilford")
	inst.components.stache:SetRate(TUNING.WILSON_HUNGER_RATE * 35)
end

local function AverageWilford(inst)
	inst.WilfordState = "Average"
	inst.components.health:StartRegen(0, 5)
	inst.components.hunger:SetRate(TUNING.WILSON_HUNGER_RATE * 1.02)
	inst.components.sanity.neg_aura_mult = (0.8)
    inst.components.sanity.night_drain_mult = (0.83)
	inst.components.health.absorb = (0.05)
	inst.components.combat.damagemultiplier = (1.1)
	inst.components.locomotor.walkspeed = (TUNING.WILSON_WALK_SPEED * 1.1)
	inst.components.locomotor.runspeed = (TUNING.WILSON_RUN_SPEED * 1.1)
	inst.Transform:SetScale(1.1,1.1,1.1)
	inst.components.eater:SetOmnivore()
	inst.AnimState:SetBuild("wilford")
	inst.components.stache:SetRate(TUNING.WILSON_HUNGER_RATE * 0.4)
end

local function PauseTheStache(inst)
	inst.StacheTimerGo = inst:DoPeriodicTask(1, function()
		inst.StacheTimer = inst.StacheTimer - 1
		if inst.StacheTimer == 0 then
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_RESTED)
			inst.StacheTimerGo:Cancel()
			inst.StacheTimerGo = nil
		end
	end)
end

local function SwitchPowers(inst)
	if inst.components.stache.current >= 100 or inst.WilfordState == "Mighty" then
		if inst.WilfordState == "Average" then
			if inst.StacheTimer == 0 then
				MightyWilford(inst)
				SpawnPrefab("shirtrip").Transform:SetPosition(inst:GetPosition():Get())
				if SwearControl == true then
					inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.WILFORD))
				else
					inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.WILFORD_SLANG))
				end
			else
				if inst.StacheTimer >= 21 then
					inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_TIMER_1)
				elseif inst.StacheTimer >= 11 and inst.StacheTimer <= 20 then
					inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_TIMER_2)
				elseif inst.StacheTimer >= 1 and inst.StacheTimer <= 10 then
					inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_TIMER_3)
				end
			end
		else
			AverageWilford(inst)
			inst.StacheTimer = inst.StacheTimer + 30
			PauseTheStache(inst)
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_BACKTONORMAL)
		end
	else
		inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_LOWPOWER)
	end
end

local function onsave(inst, data)
	data.StacheTimer = inst.StacheTimer
	data.WilfordState = inst.WilfordState
	data.StartedWorld = inst.StartedWorld
	data.StacheBooster = inst.StacheBooster
	data.HelpShown = inst.HelpShown
end

local function onload(inst, data)
	if data then
		--v1.9 or older savedata first
		if data.average then
			AverageWilford(inst)
		elseif data.mighty then
			MightyWilford(inst)
			SpawnPrefab("shirtrip").Transform:SetPosition(inst:GetPosition():Get())
		end
		if data.startedworld then
			inst.StartedWorld = true
		end
		
		--now load new savedata values
		inst.WilfordState = data.WilfordState or "Average"
		local ApplyState = {}
		ApplyState["Mighty"] = function()
		MightyWilford(inst)
		SpawnPrefab("shirtrip").Transform:SetPosition(inst:GetPosition():Get())
		end
		ApplyState["Average"] = AverageWilford
		ApplyState[inst.WilfordState](inst)
		inst.StacheTimer = data.StacheTimer or 0
		if inst.StacheTimer > 0 then
			PauseTheStache(inst)
		end
		inst.StartedWorld = data.StartedWorld or false
		inst.StacheBooster = data.StacheBooster or 0
		inst.HelpShown = data.HelpShown or false
		if inst.HelpShown == false then
			inst:DoTaskInTime(1, function() TheFrontEnd:PushScreen(MarkimodHelpScreen()) end)
		end
	end
end

local function getstatus(inst)
	if inst.WilfordState then
		return string.upper(inst.WilfordState)
	end
end

local fn = function(inst)
	inst.StacheTimer = 0
	inst.SwitchPowers = SwitchPowers
	inst.WilfordState = "Average"
	inst.StartedWorld = false
	inst.StacheBooster = 0
	
	inst:DoPeriodicTask(15, function()
		local ChanceToTalk = math.random(1,100)
		if ChanceToTalk <= 5 then
			if SwearControl == true then
				inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.WILFORD))
			else
				inst.components.talker:Say(GetRandomItem(STRINGS.MARKIMODSTRINGS.WILFORD_SLANG))
			end
		end
	end)
	
	inst.HelpShown = false
	inst.OnSave = onsave
	inst.OnLoad = onload
	
	inst:AddTag("wilford_warfstache")
	inst.soundsname = "wilford"
	inst.entity:AddMiniMapEntity()
    inst.MiniMapEntity:SetIcon("wilford.tex")
	inst:SetStateGraph("SGwilson")
	inst.AnimState:SetBank("wilson")
	inst:AddComponent("stache")
	inst.components.talker.colour = Vector3(255/255,161/255,161/255)
    inst.components.health:SetMaxHealth(200)
	inst.components.hunger:SetMax(175)
	inst.components.sanity:SetMax(150)
	inst.components.stache:SetMax(250)
	inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
	
	AverageWilford(inst)
	
	inst:DoPeriodicTask(0.1, function()
		local W = inst.WilfordState
		local S = inst.components.stache.current
		if W == "Average" then
			if S > 200 then
				inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * 0.8)
			elseif S < 40 then
				inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * -0.5)
			else
				inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * 0.2)
			end
		elseif W == "Mighty" then
			if S > 200 then
				inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * 2)
			else
				inst.components.sanity.dapperness = (TUNING.DAPPERNESS_MED * 1.5)
			end
		end
	end)
	
	inst:DoPeriodicTask(1, function()
		if inst.components.stache.current <= 50 and inst.WilfordState == "Mighty" then
			AverageWilford(inst)
			inst.StacheTimer = inst.StacheTimer + 30
			PauseTheStache(inst)
			inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.WILFORD_WORNOUT)
		end
	end)

	inst:DoPeriodicTask(0.5, function()
		if inst.StacheBooster == 1 then
			inst.components.stache:DoDelta(0.13)
		elseif inst.StacheBooster == 2 then
			inst.components.stache:DoDelta(0.2)
		elseif inst.StacheBooster == 0 then
			inst.components.stache:DoDelta(0)
		end
	end)
	
	inst:DoPeriodicTask(0.1, function()
		if inst.HUD then
			if inst.StacheTimer > 0 then
				inst.HUD.controls.status.stache.anim:GetAnimState():SetMultColour(90/255, 100/255, 170/255, 1)
			elseif inst.StacheTimer == 0 then
				inst.HUD.controls.status.stache.anim:GetAnimState():SetMultColour(1, 1, 1, 1)
			end
		end
	end)
	
	inst:DoTaskInTime(0.1, function()
		if inst.StartedWorld == false then
			inst.components.health.currenthealth = 75
			inst.components.stache.current = 50
			local SpawnFlower = 1
			local SpawnBug = 1
			repeat
				local theta = math.random() * 2 * PI
				local pt = inst:GetPosition()
				local radius = math.random(5, 10)
				local offset = FindWalkableOffset(pt, theta, radius, 12, true)
				if offset then
					local warf = SpawnPrefab("warflower")
					local pos = pt + offset
					warf.Transform:SetPosition(pos:Get())
					SpawnFlower = SpawnFlower + 1
				end
			until SpawnFlower > 4
			repeat
				local theta = math.random() * 2 * PI
				local pt = inst:GetPosition()
				local radius = math.random(5, 10)
				local offset = FindWalkableOffset(pt, theta, radius, 12, true)
				if offset then
					local warf = SpawnPrefab("warfbug")
					local pos = pt + offset
					warf.Transform:SetPosition(pos:Get())
					SpawnBug = SpawnBug + 1
				end
			until SpawnBug > 3
			inst.StartedWorld = true
		end
	end)
end

return MakePlayerCharacter("wilford", prefabs, assets, fn , start_inv)