local Screen = require "widgets/screen"
local Button = require "widgets/button"
local AnimButton = require "widgets/animbutton"
local Menu = require "widgets/menu"
local Text = require "widgets/text"
local Image = require "widgets/image"
local UIAnim = require "widgets/uianim"
local Widget = require "widgets/widget"

local MarkimodHelpScreen = Class(Screen, function(self, title, text, buttons, texthalign, additionaltext, textsize, timeout)
	Screen._ctor(self, "MarkimodHelpScreen")
	if not IsPaused() then SetPause(true) end
    TheInputProxy:SetCursorVisible(true)

	self.bg = self:AddChild(Image("images/ui.xml", "bg_plain.tex"))
    self.bg:SetTint(250/255,180/255,180/255, 0.6)
    self.bg:SetVRegPoint(ANCHOR_MIDDLE)
    self.bg:SetHRegPoint(ANCHOR_MIDDLE)
    self.bg:SetVAnchor(ANCHOR_MIDDLE)
    self.bg:SetHAnchor(ANCHOR_MIDDLE)
    self.bg:SetScaleMode(SCALEMODE_FILLSCREEN)
	
	self.root = self:AddChild(Widget("ROOT"))
    self.root:SetVAnchor(ANCHOR_MIDDLE)
    self.root:SetHAnchor(ANCHOR_MIDDLE)
    self.root:SetPosition(0,0,0)
    self.root:SetScaleMode(SCALEMODE_PROPORTIONAL)
	
    self.title = self.root:AddChild(Text(TITLEFONT, 50))
    self.title:SetPosition(0, 170, 0)
    self.title:SetString("The Markimod Collab Team needs your help!\n(Please read, extremely important!)")

	local defaulttextsize = 24
	if textsize then
		defaulttextsize = textsize
	end

    self.text = self.root:AddChild(Text(BODYTEXTFONT, defaulttextsize))
	self.text:SetVAlign(ANCHOR_TOP)

	if texthalign then
		self.text:SetHAlign(texthalign)
	end

    self.text:SetPosition(0, 40, 0)
    self.text:SetString("\n\nThe Markimod Collab Team is creating a HUGE custom adventure mode for the Don't Starve Markiplier Mod that breaks off from the normal ' survival ' routine and takes a new twist on action, comedy and horror!\n\nOur dream is to have cutscenes with narration/voice acting from various Youtubers that will also be featured in this mod such as LordMinion777, JackSepticEye, and Muyskerm, most of which who have agreed to help us with voice acting! How awesome is that! All we need now is Markiplier and Yamimash and our dream is complete!")
    self.text:EnableWordWrap(true)
    self.text:SetRegionSize(480*2, 200)
    
	self.additionaltext = self.root:AddChild(Text(BODYTEXTFONT, 24))
	self.additionaltext:SetVAlign(ANCHOR_TOP)
	self.additionaltext:SetPosition(0, -100, 0)
	self.additionaltext:SetString("\nIf you would like to help The Markimod Collab Team with Art, Programming, Sound Design, or anything else related to our project, feel free to contact us at markimodcollabteam@gmail.com\nThank you all for your continued support in helping make this dream a reality!")
	self.additionaltext:EnableWordWrap(true)
	self.additionaltext:SetRegionSize(480*2, 100)
	
	self.globe = self.bg:AddChild(Image("images/dontstarveglobe.xml", "dontstarveglobe.tex"))
	self.globe:SetScale(0.7)
	self.globe:SetPosition(550, -350, 0)
	self.globe:SetTint(1,1,1,1)
	self.wilford = self.globe:AddChild(Image("images/willybox.xml", "willybox.tex"))
	self.wilford:SetScale(1.3)
	self.wilford:SetPosition(-25, 300, 0)
	self.wilford:SetTint(1,1,1,1)
	
	local button_w = 200
	local space_between = 20
	local spacing = button_w + space_between

	self.menu = self.root:AddChild(Menu({{text = "OK", cb = function() SetPause(false) GetPlayer().HelpShown = true TheFrontEnd:PopScreen() end}}, 200, true))
	self.menu:SetHRegPoint(ANCHOR_MIDDLE)
	self.menu:SetPosition(0, -250, 0)
	self.default_focus = self.menu
end)

function MarkimodHelpScreen:OnUpdate( dt )
	if self.timeout then
		self.timeout.timeout = self.timeout.timeout - dt
		if self.timeout.timeout <= 0 then
			self.timeout.cb()
		end
	end
	self.bg:SetTint(250/255,180/255,180/255, 0.6)
	if not IsPaused() then SetPause(true) end
	return true
end

return MarkimodHelpScreen