require("stategraphs/commonstates")
local GIRL = STRINGS.MARKIMODSTRINGS.GOOGLEIRL_SPEECH
local trace = function() end
local function DoFoleySounds(inst)
	for k,v in pairs(inst.components.inventory.equipslots) do
		if v.components.inventoryitem and v.components.inventoryitem.foleysound then
			inst.SoundEmitter:PlaySound(v.components.inventoryitem.foleysound)
		end
	end
end

local actionhandlers = {
    ActionHandler(ACTIONS.CHOP, "work"),
    ActionHandler(ACTIONS.MINE, "work"),
}

local events = {
    CommonHandlers.OnLocomote(true, false),
    CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    CommonHandlers.OnAttack(),
	EventHandler("ontalk", function(inst, data)
        if inst.sg:HasStateTag("idle") then
			inst.sg:GoToState("talk", data.noanim)
        end
    end),
	EventHandler("doaction", function(inst, data) 
		if not inst.components.health:IsDead() and not inst.sg:HasStateTag("busy") then
			inst.sg:GoToState("work", data.target)
		end
	end),
}

local states = {
    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        onenter = function(inst, pushanim)    
            inst.Physics:Stop()
            if not inst.components.inventoryitem then
				inst.AnimState:SetBank("wilson")
				inst.AnimState:SetBuild("googleirl")
				inst.AnimState:PlayAnimation("idle_loop", true)
			else
				inst.AnimState:SetBank("googleirl_transform")
				inst.AnimState:SetBuild("googleirl_transform")
				inst.AnimState:PlayAnimation("collapsed")
			end
        end,
    },

    State{
        name = "run_start",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst)
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_pre")
            inst.sg.mem.foosteps = 0
        end,

        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        events = {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
        
        timeline = {        
            TimeEvent(4*FRAMES, function(inst)
                PlayFootstep(inst)
				DoFoleySounds(inst)
            end),
        },        
        
    },

    State{
        name = "run",
        tags = {"moving", "running", "canrotate"},
        
        onenter = function(inst) 
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_loop")
            
        end,
        
        onupdate = function(inst)
            inst.components.locomotor:RunForward()
        end,

        timeline = {
            TimeEvent(7*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
				DoFoleySounds(inst)
            end),
            TimeEvent(15*FRAMES, function(inst)
				inst.sg.mem.foosteps = inst.sg.mem.foosteps + 1
                PlayFootstep(inst, inst.sg.mem.foosteps < 5 and 1 or .6)
				DoFoleySounds(inst)
            end),
        },
        
        events = {   
            EventHandler("animover", function(inst) inst.sg:GoToState("run") end ),        
        },
    },
	State{
    
        name = "run_stop",
        tags = {"canrotate", "idle"},
        
        onenter = function(inst) 
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("run_pst")
        end,
        
        events = {   
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),        
        },
        
    },
    State{
        name = "talk",
        tags = {"idle", "talking"},
        
        onenter = function(inst, noanim)
            inst.components.locomotor:Stop()
            if not noanim then
                inst.AnimState:PlayAnimation("dial_loop", true)
            end

			local sound_name = inst.soundsname or inst.prefab
			inst.SoundEmitter:PlaySound("markiplier_sounds/characters/markiplier/talk_LP", "talk")

            inst.sg:SetTimeout(1.5 + math.random()*.5)
        end,
        
        ontimeout = function(inst)
            inst.SoundEmitter:KillSound("talk")
            inst.sg:GoToState("idle")
        end,
        
        onexit = function(inst)
            inst.SoundEmitter:KillSound("talk")
        end,
        
        events = {EventHandler("donetalking", function(inst) inst.sg:GoToState("idle") end)},
    }, 

    State{
        name = "attack",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst)     
            inst.sg.statemem.target = inst.components.combat.target
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")
			inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
            
            if inst.components.combat.target then
                if inst.components.combat.target and inst.components.combat.target:IsValid() then
                    inst:FacePoint(Point(inst.components.combat.target.Transform:GetWorldPosition()))
                end
            end
            
        end,
        
        timeline = {
            TimeEvent(8*FRAMES, function(inst) inst.components.combat:DoAttack(inst.sg.statemem.target) inst.sg:RemoveStateTag("abouttoattack") end),
            TimeEvent(12*FRAMES, function(inst) 
                inst.sg:RemoveStateTag("busy")
            end),               
            TimeEvent(13*FRAMES, function(inst)
                if not inst.sg.statemem.slow then
                    inst.sg:RemoveStateTag("attack")
                end
            end),
            TimeEvent(24*FRAMES, function(inst)
                if inst.sg.statemem.slow then
                    inst.sg:RemoveStateTag("attack")
                end
            end),           
        },
        
        events = {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },
    },  
    State{
        name = "work",
        tags = {"working"},
        
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")
			inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
        end,
        
        timeline = {TimeEvent(13*FRAMES, function(inst)
			inst:PerformBufferedAction()
			if inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
				inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS).components.finiteuses:Use(1)
			end
		end )},
        events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)}
    },
    State{
        name = "craftchromesword",
        tags = {"crafting", "busy"},
        
        timeline = {
            TimeEvent(4*FRAMES, function( inst )
                inst.sg:RemoveStateTag("busy", "working")
            end),
        },
        onenter = function(inst)
            inst.sg:SetTimeout(1.5)
            inst.components.locomotor:Stop()
            inst.SoundEmitter:PlaySound("dontstarve/wilson/make_trap", "make")
            inst.components.talker:Say(GIRL.fixing_chromesword)
            inst.AnimState:PlayAnimation("build_pre")
            inst.AnimState:PushAnimation("build_loop", true)
        end,
        
        ontimeout = function(inst)
            inst.AnimState:PlayAnimation("build_pst")
            inst.sg:GoToState("idle", false)
            local weapon = SpawnPrefab("chromesword")
			weapon.entity:AddTransform()
			inst.components.inventory:Equip(weapon)
        end,
        
        onexit = function(inst)
            inst.SoundEmitter:KillSound("make")
        end,
        
    },
    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("death")
			inst.SoundEmitter:PlaySound("markiplier_sounds/characters/markiplier/death_voice")
			--inst.components.talker:Say(STRINGS.MARKIMODSTRINGS.AVENGEME)
			inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))
        end,

        events = {
            EventHandler("animover", function(inst)
                    inst:Remove()
            end ),
		},
    },  
   
    State{
        name = "hit",
        tags = {"busy"},
        
        onenter = function(inst)
            inst:ClearBufferedAction()
            inst.AnimState:PlayAnimation("hit")
			inst.SoundEmitter:PlaySound("markiplier_sounds/characters/markiplier/hurt")
            inst.Physics:Stop()            
        end,
        
        events = {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),
        }, 
        
        timeline = {
            TimeEvent(3*FRAMES, function(inst)
                inst.sg:RemoveStateTag("busy")
            end),
        },               
    },

    State{
        name = "stunned",
        tags = {"busy", "canrotate"},

        onenter = function(inst)
            inst:ClearBufferedAction()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle_sanity_pre")
            inst.AnimState:PushAnimation("idle_sanity_loop", true)
			inst.SoundEmitter:PlaySound("markiplier_sounds/characters/markiplier/hurt")
            inst.sg:SetTimeout(5)
        end,

        ontimeout = function(inst)
            inst.sg:GoToState("idle")
        end,
    },
	State{
		name = "transformdown",
		tags = {"busy", "transforming"},
		
		onenter = function(inst)
			inst:ClearBufferedAction()
			inst.Physics:Stop()
			inst.components.locomotor:Stop()
			inst.AnimState:SetBank("googleirl_transform")
			inst.AnimState:SetBuild("googleirl_transform")
			inst.AnimState:PlayAnimation("collapse")
		end,
		
        events = {
            EventHandler("animover", function(inst)
			inst:makeinventoryitem(inst)
			inst.sg:GoToState("idle")
		end)},
	},
	State{
		name = "transformup",
		tags = {"busy", "transforming"},
		
		onenter = function(inst)
			inst.AnimState:SetBank("googleirl_transform")
			inst.AnimState:SetBuild("googleirl_transform")
			inst.AnimState:PlayAnimation("uncollapse")
		end,
		
        events = {
            EventHandler("animover", function(inst)
			inst.AnimState:SetBank("wilson")
			inst.AnimState:SetBuild("googleirl")
			inst.sg:GoToState("idle")
		end)},
	}
}
    

return StateGraph("googleirl", states, events, "idle", actionhandlers)