require("stategraphs/commonstates")

local actionhandlers = {}

local events = {
    EventHandler("locomote", function(inst) 
		local is_moving = inst.sg:HasStateTag("moving")
		local open = inst.sg:HasStateTag("open")
        local should_move = inst.components.locomotor:WantsToMoveForward()
        if should_move and not is_moving and not open then
			inst.sg:GoToState("bounce_pre")
        end
	end),
	EventHandler("ontalk", function(inst, data)
		if not inst.sg:HasStateTag("open") and not inst.sg:HasStateTag("sleeping") and not inst.sg:HasStateTag("moving") then
			if inst.sg:HasStateTag("idle") then
				inst.sg:GoToState("talk", data.noanim)
			else
				inst.sg:GoToState("talk_noanim")
			end
		end
    end)
}

local states = {
    State{
        name = "idle",
        tags = {"idle", "canrotate"},
        
        onenter = function(inst, pushanim)
			inst.AnimState:SetBank("tinyboxtim_idle")
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle")
		end,
        
        events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)}
    },
    
	State{
        name = "open",
        tags = {"busy", "open"},
        
        onenter = function(inst)
            inst.Physics:Stop()
			inst.AnimState:SetBank("tinyboxtim_openandclose")
			inst.AnimState:PlayAnimation("pre_base")
			inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/tinyboxtim/open")
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("open_idle") end )}
    },

    State{
        name = "open_idle",
        tags = {"busy", "open"},
        
        onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_openandclose")
            inst.AnimState:PlayAnimation("loop_base")
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("open_idle") end )}
    },

    State{
        name = "close",
        tags = {"open"},
        
        onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_openandclose")
            inst.AnimState:PlayAnimation("pst_base")
			inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/tinyboxtim/close")
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end )}    
    },
	
	State{
        name = "sleep",
        tags = {"busy", "sleeping"},
        
        onenter = function(inst)
			inst.components.locomotor:StopMoving()
			inst.AnimState:SetBank("tinyboxtim_sleeping")
            inst.AnimState:PlayAnimation("pre_sleeping")
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("sleeping") end ), EventHandler("onwakeup", function(inst) inst.sg:GoToState("wakeup") end)}    
    },
	
	State{
        name = "sleeping",
        tags = {"busy", "sleeping"},
        
        onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_sleeping")
            inst.AnimState:PlayAnimation("loop_sleeping")
			if math.random(1,100) <= 40 then
				inst.components.talker:Say("zzz")
			end
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("sleeping") end ), EventHandler("onwakeup", function(inst) inst.sg:GoToState("wakeup") end)}    
    },
	
	State{
        name = "wakeup",
        tags = {"busy", "waking"},
        
        onenter = function(inst)
			inst.components.locomotor:StopMoving()
			inst.AnimState:SetBank("tinyboxtim_sleeping")
            inst.AnimState:PlayAnimation("pst_sleeping")
        end,

        events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end )}    
    },
	
	State{
		name = "bounce_pre",
		tags = {"moving", "canrotate"},
		
		onenter = function(inst) 
			inst.AnimState:SetBank("tinyboxtim_bouncing")
			inst.AnimState:PlayAnimation("pre_bouncing")
		end,
		
		events = {EventHandler("animover", function(inst) inst.sg:GoToState("bounce_mid") end )}
	},
	
	State{
		name = "bounce_mid",
		tags = {"moving"},
		
		onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_bouncing")
			inst.components.locomotor:WalkForward()
			inst.AnimState:PlayAnimation("loop_bouncing")
		end,
		
		events = {EventHandler("animover", function(inst) inst.sg:GoToState("bounce_pst") end )}
	},
	
	State{
		name = "bounce_pst",
		tags = {"moving", "canrotate"},
		
		onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_bouncing")
			inst.components.locomotor:StopMoving()
			inst.AnimState:PlayAnimation("pst_bouncing")
		end,
		
		events = { EventHandler("animover", function(inst) 
			if inst.components.locomotor:WantsToMoveForward() then
				inst.sg:GoToState("bounce_pre")
			else
				inst.sg:GoToState("idle")
			end
		end)}
	},
	
	State{
        name = "talk_pre",
        tags = {"idle", "talking"},
        
        onenter = function(inst, noanim)
			inst.Physics:Stop()
            inst.components.locomotor:Stop()
            if not noanim then
				inst.AnimState:SetBank("tinyboxtim_talking")
                inst.AnimState:PlayAnimation("pre_talking")
			else
				inst.sg:GoToState("talk_noanim")
            end
        end,
        
        events = {EventHandler("animover", function(inst) inst.sg:GoToState("talk") end)},
    },
	
	State{
		name = "talk",
		tags = {"idle", "talking"},
		
		onenter = function(inst)
			inst.Physics:Stop()
			inst.components.locomotor:Stop()
			inst.AnimState:SetBank("tinyboxtim_talking")
            inst.AnimState:PlayAnimation("loop_talking", true)
            inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/tinyboxtim/talk", "talk")
            inst.sg:SetTimeout(1.5 + math.random()*.5)
		end,
		
		ontimeout = function(inst)
            inst.SoundEmitter:KillSound("talk")
            inst.sg:GoToState("idle")
        end,
		
		onexit = function(inst)
            inst.SoundEmitter:KillSound("talk")
        end,
        
        events = {EventHandler("donetalking", function(inst) inst.sg:GoToState("talk_pst") end)}
	},
	
	State{
		name = "talk_noanim",
		tags = {"idle", "talking"},
		
		onenter = function(inst)
            inst.SoundEmitter:PlaySound("markiplier_sounds/markiplier/tinyboxtim/talk", "talk")
            inst.sg:SetTimeout(1.5 + math.random()*.5)
		end,
		
		ontimeout = function(inst)
            inst.SoundEmitter:KillSound("talk")
        end,
		
		onexit = function(inst)
            inst.SoundEmitter:KillSound("talk")
        end,
    },
	
	State{
		name = "talk_pst",
		tags = {"idle"},
		
		onenter = function(inst)
			inst.AnimState:SetBank("tinyboxtim_talking")
            inst.AnimState:PlayAnimation("pst_talking")
		end,
		
		events = {EventHandler("animover", function(inst) inst.sg:GoToState("idle") end)}
	}
}

return StateGraph("tinyboxtim", states, events, "idle", actionhandlers)